@@ -53,7 +53,7 @@ func (p AbsPath) Bytes() []byte {
 
 // Dir returns p's directory.
 func (p AbsPath) Dir() AbsPath {
-	return NewAbsPath(path.Dir(p.absPath))
+	return NewAbsPath(filepath.Dir(p.absPath)).ToSlash()
 }
 
 // Empty returns if p is empty.
@@ -147,11 +147,11 @@ func (p AbsPath) TrimDirPrefix(dirPrefixAbsPath AbsPath) (RelPath, error) {
 		return EmptyRelPath, nil
 	}
 	dirAbsPath := dirPrefixAbsPath
-	if dirAbsPath.absPath != "/" {
+	if !strings.HasSuffix(dirAbsPath.absPath, "/") {
 		dirAbsPath.absPath += "/"
 	}
 	if !strings.HasPrefix(p.absPath, dirAbsPath.absPath) {
-		return EmptyRelPath, &notInAbsDirError{
+		return EmptyRelPath, &NotInAbsDirError{
 			pathAbsPath: p,
 			dirAbsPath:  dirPrefixAbsPath,
 		}
@@ -159,6 +159,11 @@ func (p AbsPath) TrimDirPrefix(dirPrefixAbsPath AbsPath) (RelPath, error) {
 	return NewRelPath(p.absPath[len(dirAbsPath.absPath):]), nil
 }
 
+// TrimSuffix returns p with the optional suffix removed.
+func (p AbsPath) TrimSuffix(suffix string) AbsPath {
+	return NewAbsPath(strings.TrimSuffix(p.absPath, suffix))
+}
+
 // Type implements github.com/spf13/pflag.Value.Type.
 func (p AbsPath) Type() string {
 	return "path"
@@ -185,7 +190,7 @@ func HomeDirAbsPath() (AbsPath, error) {
 // StringToAbsPathHookFunc is a github.com/mitchellh/mapstructure.DecodeHookFunc
 // that parses an AbsPath from a string.
 func StringToAbsPathHookFunc() mapstructure.DecodeHookFunc {
-	return func(from, to reflect.Type, data interface{}) (interface{}, error) {
+	return func(from, to reflect.Type, data any) (any, error) {
 		if to != reflect.TypeOf(EmptyAbsPath) {
 			return data, nil
 		}
