@@ -2,7 +2,6 @@ package chezmoi
 
 // FIXME add builtin support for --passphrase
 // FIXME add builtin support for --symmetric
-// FIXME add builtin support for SSH keys if recommended
 
 import (
 	"bytes"
@@ -20,29 +19,27 @@ import (
 // An AgeEncryption uses age for encryption and decryption. See
 // https://age-encryption.org.
 type AgeEncryption struct {
-	UseBuiltin      bool
-	BaseSystem      System
-	Command         string
-	Args            []string
-	Identity        AbsPath
-	Identities      []AbsPath
-	Passphrase      bool
-	Recipient       string
-	Recipients      []string
-	RecipientsFile  AbsPath
-	RecipientsFiles []AbsPath
-	Suffix          string
-	Symmetric       bool
+	UseBuiltin      bool      `json:"useBuiltin"      mapstructure:"useBuiltin"      yaml:"useBuiltin"`
+	Command         string    `json:"command"         mapstructure:"command"         yaml:"command"`
+	Args            []string  `json:"args"            mapstructure:"args"            yaml:"args"`
+	Identity        AbsPath   `json:"identity"        mapstructure:"identity"        yaml:"identity"`
+	Identities      []AbsPath `json:"identities"      mapstructure:"identities"      yaml:"identities"`
+	Passphrase      bool      `json:"passphrase"      mapstructure:"passphrase"      yaml:"passphrase"`
+	Recipient       string    `json:"recipient"       mapstructure:"recipient"       yaml:"recipient"`
+	Recipients      []string  `json:"recipients"      mapstructure:"recipients"      yaml:"recipients"`
+	RecipientsFile  AbsPath   `json:"recipientsFile"  mapstructure:"recipientsFile"  yaml:"recipientsFile"`
+	RecipientsFiles []AbsPath `json:"recipientsFiles" mapstructure:"recipientsFiles" yaml:"recipientsFiles"`
+	Suffix          string    `json:"suffix"          mapstructure:"suffix"          yaml:"suffix"`
+	Symmetric       bool      `json:"symmetric"       mapstructure:"symmetric"       yaml:"symmetric"`
 }
 
-// Decrypt implements Encyrption.Decrypt.
+// Decrypt implements Encryption.Decrypt.
 func (e *AgeEncryption) Decrypt(ciphertext []byte) ([]byte, error) {
 	if e.UseBuiltin {
 		return e.builtinDecrypt(ciphertext)
 	}
 
-	//nolint:gosec
-	cmd := exec.Command(e.Command, append(e.decryptArgs(), e.Args...)...)
+	cmd := exec.Command(e.Command, append(e.decryptArgs(), e.Args...)...) //nolint:gosec
 	cmd.Stdin = bytes.NewReader(ciphertext)
 	cmd.Stderr = os.Stderr
 	return chezmoilog.LogCmdOutput(cmd)
@@ -55,11 +52,11 @@ func (e *AgeEncryption) DecryptToFile(plaintextAbsPath AbsPath, ciphertext []byt
 		if err != nil {
 			return err
 		}
-		return e.BaseSystem.WriteFile(plaintextAbsPath, plaintext, 0o644)
+		return os.WriteFile(plaintextAbsPath.String(), plaintext, 0o644) //nolint:gosec
 	}
 
-	//nolint:gosec
-	cmd := exec.Command(e.Command, append(append(e.decryptArgs(), "--output", plaintextAbsPath.String()), e.Args...)...)
+	args := append(append(e.decryptArgs(), "--output", plaintextAbsPath.String()), e.Args...)
+	cmd := exec.Command(e.Command, args...) //nolint:gosec
 	cmd.Stdin = bytes.NewReader(ciphertext)
 	cmd.Stderr = os.Stderr
 	return chezmoilog.LogCmdRun(cmd)
@@ -71,8 +68,7 @@ func (e *AgeEncryption) Encrypt(plaintext []byte) ([]byte, error) {
 		return e.builtinEncrypt(plaintext)
 	}
 
-	//nolint:gosec
-	cmd := exec.Command(e.Command, append(e.encryptArgs(), e.Args...)...)
+	cmd := exec.Command(e.Command, append(e.encryptArgs(), e.Args...)...) //nolint:gosec
 	cmd.Stdin = bytes.NewReader(plaintext)
 	cmd.Stderr = os.Stderr
 	return chezmoilog.LogCmdOutput(cmd)
@@ -81,15 +77,16 @@ func (e *AgeEncryption) Encrypt(plaintext []byte) ([]byte, error) {
 // EncryptFile implements Encryption.EncryptFile.
 func (e *AgeEncryption) EncryptFile(plaintextAbsPath AbsPath) ([]byte, error) {
 	if e.UseBuiltin {
-		plaintext, err := e.BaseSystem.ReadFile(plaintextAbsPath)
+		plaintext, err := os.ReadFile(plaintextAbsPath.String())
 		if err != nil {
 			return nil, err
 		}
 		return e.builtinEncrypt(plaintext)
 	}
 
-	//nolint:gosec
-	cmd := exec.Command(e.Command, append(append(e.encryptArgs(), e.Args...), plaintextAbsPath.String())...)
+	cmd := exec.Command( //nolint:gosec
+		e.Command,
+		append(append(e.encryptArgs(), e.Args...), plaintextAbsPath.String())...)
 	cmd.Stderr = os.Stderr
 	return chezmoilog.LogCmdOutput(cmd)
 }
@@ -110,10 +107,10 @@ func (e *AgeEncryption) builtinDecrypt(ciphertext []byte) ([]byte, error) {
 		return nil, err
 	}
 	buffer := &bytes.Buffer{}
-	if _, err = io.Copy(buffer, r); err != nil {
+	if _, err := io.Copy(buffer, r); err != nil {
 		return nil, err
 	}
-	return buffer.Bytes(), err
+	return buffer.Bytes(), nil
 }
 
 // builtinEncrypt encrypts ciphertext using the builtin age.
@@ -124,14 +121,14 @@ func (e *AgeEncryption) builtinEncrypt(plaintext []byte) ([]byte, error) {
 	}
 	output := &bytes.Buffer{}
 	armorWriter := armor.NewWriter(output)
-	writer, err := age.Encrypt(armorWriter, recipients...)
+	writeCloser, err := age.Encrypt(armorWriter, recipients...)
 	if err != nil {
 		return nil, err
 	}
-	if _, err := io.Copy(writer, bytes.NewReader(plaintext)); err != nil {
+	if _, err := io.Copy(writeCloser, bytes.NewReader(plaintext)); err != nil {
 		return nil, err
 	}
-	if err := writer.Close(); err != nil {
+	if err := writeCloser.Close(); err != nil {
 		return nil, err
 	}
 	if err := armorWriter.Close(); err != nil {
@@ -247,16 +244,14 @@ func (e *AgeEncryption) identityArgs() []string {
 	return args
 }
 
-// parseIdentityFile parses the identities from indentityFile using the builtin
+// parseIdentityFile parses the identities from identityFile using the builtin
 // age.
 func parseIdentityFile(identityFile AbsPath) (identities []age.Identity, err error) {
 	var file *os.File
 	if file, err = os.Open(identityFile.String()); err != nil {
 		return
 	}
-	defer func() {
-		err = multierr.Append(err, file.Close())
-	}()
+	defer multierr.AppendInvoke(&err, multierr.Close(file))
 	identities, err = age.ParseIdentities(file)
 	return
 }
@@ -268,9 +263,7 @@ func parseRecipientsFile(recipientsFile AbsPath) (recipients []age.Recipient, er
 	if file, err = os.Open(recipientsFile.String()); err != nil {
 		return
 	}
-	defer func() {
-		err = multierr.Append(err, file.Close())
-	}()
+	defer multierr.AppendInvoke(&err, multierr.Close(file))
 	recipients, err = age.ParseRecipients(file)
 	return
 }
