@@ -11,11 +11,14 @@ func (NullPersistentState) Close() error { return nil }
 func (NullPersistentState) CopyTo(s PersistentState) error { return nil }
 
 // Data does nothing.
-func (NullPersistentState) Data() (interface{}, error) { return nil, nil }
+func (NullPersistentState) Data() (any, error) { return nil, nil }
 
 // Delete does nothing.
 func (NullPersistentState) Delete(bucket, key []byte) error { return nil }
 
+// DeleteBucket does nothing.
+func (NullPersistentState) DeleteBucket(bucket []byte) error { return nil }
+
 // ForEach does nothing.
 func (NullPersistentState) ForEach(bucket []byte, fn func(k, v []byte) error) error { return nil }
 
