@@ -5,9 +5,10 @@ import (
 	"io/fs"
 	"os"
 	"os/exec"
+	"path/filepath"
 	"runtime"
+	"time"
 
-	"github.com/bmatcuk/doublestar/v4"
 	vfs "github.com/twpayne/go-vfs/v4"
 	"go.uber.org/multierr"
 
@@ -17,19 +18,14 @@ import (
 // A RealSystemOption sets an option on a RealSystem.
 type RealSystemOption func(*RealSystem)
 
+// Chtimes implements System.Chtimes.
+func (s *RealSystem) Chtimes(name AbsPath, atime, mtime time.Time) error {
+	return s.fileSystem.Chtimes(name.String(), atime, mtime)
+}
+
 // Glob implements System.Glob.
 func (s *RealSystem) Glob(pattern string) ([]string, error) {
-	return doublestar.Glob(s.UnderlyingFS(), pattern)
-}
-
-// IdempotentCmdCombinedOutput implements System.IdempotentCmdCombinedOutput.
-func (s *RealSystem) IdempotentCmdCombinedOutput(cmd *exec.Cmd) ([]byte, error) {
-	return chezmoilog.LogCmdCombinedOutput(cmd)
-}
-
-// IdempotentCmdOutput implements System.IdempotentCmdOutput.
-func (s *RealSystem) IdempotentCmdOutput(cmd *exec.Cmd) ([]byte, error) {
-	return chezmoilog.LogCmdOutput(cmd)
+	return Glob(s.UnderlyingFS(), filepath.ToSlash(pattern))
 }
 
 // Link implements System.Link.
@@ -66,6 +62,11 @@ func (s *RealSystem) ReadFile(name AbsPath) ([]byte, error) {
 	return s.fileSystem.ReadFile(name.String())
 }
 
+// Remove implements System.Remove.
+func (s *RealSystem) Remove(name AbsPath) error {
+	return s.fileSystem.Remove(name.String())
+}
+
 // RemoveAll implements System.RemoveAll.
 func (s *RealSystem) RemoveAll(name AbsPath) error {
 	return s.fileSystem.RemoveAll(name.String())
@@ -81,16 +82,27 @@ func (s *RealSystem) RunCmd(cmd *exec.Cmd) error {
 	return chezmoilog.LogCmdRun(cmd)
 }
 
-// RunIdempotentCmd implements System.RunIdempotentCmd.
-func (s *RealSystem) RunIdempotentCmd(cmd *exec.Cmd) error {
-	return chezmoilog.LogCmdRun(cmd)
-}
-
 // RunScript implements System.RunScript.
-func (s *RealSystem) RunScript(scriptname RelPath, dir AbsPath, data []byte, interpreter *Interpreter) (err error) {
+func (s *RealSystem) RunScript(
+	scriptname RelPath,
+	dir AbsPath,
+	data []byte,
+	options RunScriptOptions,
+) (err error) {
+	// Create the script temporary directory, if needed.
+	s.createScriptTempDirOnce.Do(func() {
+		if !s.scriptTempDir.Empty() {
+			err = os.MkdirAll(s.scriptTempDir.String(), 0o700)
+		}
+	})
+	if err != nil {
+		return err
+	}
+
 	// Write the temporary script file. Put the randomness at the front of the
 	// filename to preserve any file extension for Windows scripts.
-	f, err := os.CreateTemp("", "*."+scriptname.Base())
+	var f *os.File
+	f, err = os.CreateTemp(s.scriptTempDir.String(), "*."+scriptname.Base())
 	if err != nil {
 		return
 	}
@@ -111,11 +123,12 @@ func (s *RealSystem) RunScript(scriptname RelPath, dir AbsPath, data []byte, int
 		return
 	}
 
-	cmd := interpreter.ExecCommand(f.Name())
+	cmd := options.Interpreter.ExecCommand(f.Name())
 	cmd.Dir, err = s.getScriptWorkingDir(dir)
 	if err != nil {
 		return err
 	}
+	cmd.Env = s.scriptEnv
 	cmd.Stdin = os.Stdin
 	cmd.Stdout = os.Stdout
 	cmd.Stderr = os.Stderr
@@ -123,6 +136,11 @@ func (s *RealSystem) RunScript(scriptname RelPath, dir AbsPath, data []byte, int
 	return s.RunCmd(cmd)
 }
 
+// SetScriptEnv sets the environment variables for scripts.
+func (s *RealSystem) SetScriptEnv(scriptEnv []string) {
+	s.scriptEnv = scriptEnv
+}
+
 // Stat implements System.Stat.
 func (s *RealSystem) Stat(name AbsPath) (fs.FileInfo, error) {
 	return s.fileSystem.Stat(name.String())
@@ -133,10 +151,15 @@ func (s *RealSystem) UnderlyingFS() vfs.FS {
 	return s.fileSystem
 }
 
+// UnderlyingSystem implements System.UnderlyingSystem.
+func (s *RealSystem) UnderlyingSystem() System {
+	return s
+}
+
 // getScriptWorkingDir returns the script's working directory.
 //
 // If this is a before_ script then the requested working directory may not
-// actually exist yet, so search through the parent directory hierarchy till
+// actually exist yet, so search through the parent directory hierarchy until
 // we find a suitable working directory.
 func (s *RealSystem) getScriptWorkingDir(dir AbsPath) (string, error) {
 	// This should always terminate because dir will eventually become ".", i.e.
