@@ -1,21 +1,29 @@
 package chezmoi
 
 import (
+	"context"
 	"errors"
 	"io/fs"
 	"os/exec"
 	"sort"
+	"strings"
+	"time"
 
 	vfs "github.com/twpayne/go-vfs/v4"
+	"golang.org/x/sync/errgroup"
 )
 
-// A System reads from and writes to a filesystem, executes idempotent commands,
-// runs scripts, and persists state.
-type System interface {
+type RunScriptOptions struct {
+	Interpreter *Interpreter
+	Condition   ScriptCondition
+}
+
+// A System reads from and writes to a filesystem, runs scripts, and persists
+// state.
+type System interface { //nolint:interfacebloat
 	Chmod(name AbsPath, mode fs.FileMode) error
+	Chtimes(name AbsPath, atime, mtime time.Time) error
 	Glob(pattern string) ([]string, error)
-	IdempotentCmdCombinedOutput(cmd *exec.Cmd) ([]byte, error)
-	IdempotentCmdOutput(cmd *exec.Cmd) ([]byte, error)
 	Link(oldname, newname AbsPath) error
 	Lstat(filename AbsPath) (fs.FileInfo, error)
 	Mkdir(name AbsPath, perm fs.FileMode) error
@@ -23,13 +31,14 @@ type System interface {
 	ReadDir(name AbsPath) ([]fs.DirEntry, error)
 	ReadFile(name AbsPath) ([]byte, error)
 	Readlink(name AbsPath) (string, error)
+	Remove(name AbsPath) error
 	RemoveAll(name AbsPath) error
 	Rename(oldpath, newpath AbsPath) error
 	RunCmd(cmd *exec.Cmd) error
-	RunIdempotentCmd(cmd *exec.Cmd) error
-	RunScript(scriptname RelPath, dir AbsPath, data []byte, interpreter *Interpreter) error
+	RunScript(scriptname RelPath, dir AbsPath, data []byte, options RunScriptOptions) error
 	Stat(name AbsPath) (fs.FileInfo, error)
 	UnderlyingFS() vfs.FS
+	UnderlyingSystem() System
 	WriteFile(filename AbsPath, data []byte, perm fs.FileMode) error
 	WriteSymlink(oldname string, newname AbsPath) error
 }
@@ -37,17 +46,14 @@ type System interface {
 // A emptySystemMixin simulates an empty system.
 type emptySystemMixin struct{}
 
-func (emptySystemMixin) Glob(pattern string) ([]string, error)                     { return nil, nil }
-func (emptySystemMixin) IdempotentCmdCombinedOutput(cmd *exec.Cmd) ([]byte, error) { return nil, nil }
-func (emptySystemMixin) IdempotentCmdOutput(cmd *exec.Cmd) ([]byte, error)         { return nil, nil }
-func (emptySystemMixin) Lstat(name AbsPath) (fs.FileInfo, error)                   { return nil, fs.ErrNotExist }
-func (emptySystemMixin) RawPath(path AbsPath) (AbsPath, error)                     { return path, nil }
-func (emptySystemMixin) ReadDir(name AbsPath) ([]fs.DirEntry, error)               { return nil, fs.ErrNotExist }
-func (emptySystemMixin) ReadFile(name AbsPath) ([]byte, error)                     { return nil, fs.ErrNotExist }
-func (emptySystemMixin) Readlink(name AbsPath) (string, error)                     { return "", fs.ErrNotExist }
-func (emptySystemMixin) RunIdempotentCmd(cmd *exec.Cmd) error                      { return nil }
-func (emptySystemMixin) Stat(name AbsPath) (fs.FileInfo, error)                    { return nil, fs.ErrNotExist }
-func (emptySystemMixin) UnderlyingFS() vfs.FS                                      { return nil }
+func (emptySystemMixin) Glob(pattern string) ([]string, error)       { return nil, nil }
+func (emptySystemMixin) Lstat(name AbsPath) (fs.FileInfo, error)     { return nil, fs.ErrNotExist }
+func (emptySystemMixin) RawPath(path AbsPath) (AbsPath, error)       { return path, nil }
+func (emptySystemMixin) ReadDir(name AbsPath) ([]fs.DirEntry, error) { return nil, fs.ErrNotExist }
+func (emptySystemMixin) ReadFile(name AbsPath) ([]byte, error)       { return nil, fs.ErrNotExist }
+func (emptySystemMixin) Readlink(name AbsPath) (string, error)       { return "", fs.ErrNotExist }
+func (emptySystemMixin) Stat(name AbsPath) (fs.FileInfo, error)      { return nil, fs.ErrNotExist }
+func (emptySystemMixin) UnderlyingFS() vfs.FS                        { return nil }
 
 // A noUpdateSystemMixin panics on any update.
 type noUpdateSystemMixin struct{}
@@ -56,6 +62,10 @@ func (noUpdateSystemMixin) Chmod(name AbsPath, perm fs.FileMode) error {
 	panic("update to no update system")
 }
 
+func (noUpdateSystemMixin) Chtimes(name AbsPath, atime, mtime time.Time) error {
+	panic("update to no update system")
+}
+
 func (noUpdateSystemMixin) Link(oldname, newname AbsPath) error {
 	panic("update to no update system")
 }
@@ -64,6 +74,10 @@ func (noUpdateSystemMixin) Mkdir(name AbsPath, perm fs.FileMode) error {
 	panic("update to no update system")
 }
 
+func (noUpdateSystemMixin) Remove(name AbsPath) error {
+	panic("update to no update system")
+}
+
 func (noUpdateSystemMixin) RemoveAll(name AbsPath) error {
 	panic("update to no update system")
 }
@@ -76,7 +90,12 @@ func (noUpdateSystemMixin) RunCmd(cmd *exec.Cmd) error {
 	panic("update to no update system")
 }
 
-func (noUpdateSystemMixin) RunScript(scriptname RelPath, dir AbsPath, data []byte, interpreter *Interpreter) error {
+func (noUpdateSystemMixin) RunScript(
+	scriptname RelPath,
+	dir AbsPath,
+	data []byte,
+	options RunScriptOptions,
+) error {
 	panic("update to no update system")
 }
 
@@ -134,8 +153,8 @@ func MkdirAll(system System, absPath AbsPath, perm fs.FileMode) error {
 // A WalkFunc is called for every entry in a directory.
 type WalkFunc func(absPath AbsPath, fileInfo fs.FileInfo, err error) error
 
-// Walk walks rootAbsPath in system, alling walkFunc for each file or directory in
-// the tree, including rootAbsPath.
+// Walk walks rootAbsPath in system, calling walkFunc for each file or directory
+// in the tree, including rootAbsPath.
 //
 // Walk does not follow symlinks.
 func Walk(system System, rootAbsPath AbsPath, walkFunc WalkFunc) error {
@@ -145,9 +164,9 @@ func Walk(system System, rootAbsPath AbsPath, walkFunc WalkFunc) error {
 	return vfs.Walk(system.UnderlyingFS(), rootAbsPath.String(), outerWalkFunc)
 }
 
-// A WalkSourceDirFunc is a function called for every entry in a source
-// directory.
-type WalkSourceDirFunc func(absPath AbsPath, fileInfo fs.FileInfo, err error) error
+// A concurrentWalkSourceDirFunc is a function called concurrently for every
+// entry in a source directory.
+type concurrentWalkSourceDirFunc func(ctx context.Context, absPath AbsPath, fileInfo fs.FileInfo, err error) error
 
 // WalkSourceDir walks the source directory rooted at sourceDirAbsPath in
 // system, calling walkFunc for each file or directory in the tree, including
@@ -159,15 +178,15 @@ type WalkSourceDirFunc func(absPath AbsPath, fileInfo fs.FileInfo, err error) er
 // Directory entries .chezmoidata.<format> and .chezmoitemplates are visited
 // before all other entries. All other entries are visited in alphabetical
 // order.
-func WalkSourceDir(system System, sourceDirAbsPath AbsPath, walkFunc WalkSourceDirFunc) error {
+func WalkSourceDir(system System, sourceDirAbsPath AbsPath, walkFunc WalkFunc) error {
 	fileInfo, err := system.Stat(sourceDirAbsPath)
 	if err != nil {
 		err = walkFunc(sourceDirAbsPath, nil, err)
 	} else {
 		err = walkSourceDir(system, sourceDirAbsPath, fileInfo, walkFunc)
-	}
-	if errors.Is(err, fs.SkipDir) {
-		return nil
+		if errors.Is(err, fs.SkipDir) {
+			err = nil
+		}
 	}
 	return err
 }
@@ -180,11 +199,11 @@ var sourceDirEntryOrder = map[string]int{
 	dataName + ".json": -2,
 	dataName + ".toml": -2,
 	dataName + ".yaml": -2,
-	templatesDirName:   -1,
+	TemplatesDirName:   -1,
 }
 
 // walkSourceDir is a helper function for WalkSourceDir.
-func walkSourceDir(system System, name AbsPath, fileInfo fs.FileInfo, walkFunc WalkSourceDirFunc) error {
+func walkSourceDir(system System, name AbsPath, fileInfo fs.FileInfo, walkFunc WalkFunc) error {
 	switch err := walkFunc(name, fileInfo, nil); {
 	case fileInfo.IsDir() && errors.Is(err, fs.SkipDir):
 		return nil
@@ -202,6 +221,78 @@ func walkSourceDir(system System, name AbsPath, fileInfo fs.FileInfo, walkFunc W
 		}
 	}
 
+	sortSourceDirEntries(dirEntries)
+
+	for _, dirEntry := range dirEntries {
+		fileInfo, err := dirEntry.Info()
+		if err != nil {
+			err = walkFunc(name, nil, err)
+			if err != nil {
+				return err
+			}
+		}
+		if err := walkSourceDir(system, name.JoinString(dirEntry.Name()), fileInfo, walkFunc); err != nil {
+			if !errors.Is(err, fs.SkipDir) {
+				return err
+			}
+		}
+	}
+
+	return nil
+}
+
+func concurrentWalkSourceDir(
+	ctx context.Context, system System, dirAbsPath AbsPath, walkFunc concurrentWalkSourceDirFunc,
+) error {
+	dirEntries, err := system.ReadDir(dirAbsPath)
+	if err != nil {
+		return walkFunc(ctx, dirAbsPath, nil, err)
+	}
+	sortSourceDirEntries(dirEntries)
+
+	// Walk all control plane entries in order.
+	visitDirEntry := func(dirEntry fs.DirEntry) error {
+		absPath := dirAbsPath.Join(NewRelPath(dirEntry.Name()))
+		fileInfo, err := dirEntry.Info()
+		if err != nil {
+			return walkFunc(ctx, absPath, nil, err)
+		}
+		switch err := walkFunc(ctx, absPath, fileInfo, nil); {
+		case fileInfo.IsDir() && errors.Is(err, fs.SkipDir):
+			return nil
+		case err != nil:
+			return err
+		case fileInfo.IsDir():
+			return concurrentWalkSourceDir(ctx, system, absPath, walkFunc)
+		default:
+			return nil
+		}
+	}
+	i := 0
+	for ; i < len(dirEntries); i++ {
+		dirEntry := dirEntries[i]
+		if !strings.HasPrefix(dirEntry.Name(), ".") {
+			break
+		}
+		if err := visitDirEntry(dirEntry); err != nil {
+			return err
+		}
+	}
+
+	// Walk all remaining entries concurrently.
+	visitDirEntryFunc := func(dirEntry fs.DirEntry) func() error {
+		return func() error {
+			return visitDirEntry(dirEntry)
+		}
+	}
+	group, ctx := errgroup.WithContext(ctx)
+	for _, dirEntry := range dirEntries[i:] {
+		group.Go(visitDirEntryFunc(dirEntry))
+	}
+	return group.Wait()
+}
+
+func sortSourceDirEntries(dirEntries []fs.DirEntry) {
 	sort.Slice(dirEntries, func(i, j int) bool {
 		nameI := dirEntries[i].Name()
 		nameJ := dirEntries[j].Name()
@@ -216,22 +307,4 @@ func walkSourceDir(system System, name AbsPath, fileInfo fs.FileInfo, walkFunc W
 			return false
 		}
 	})
-
-	for _, dirEntry := range dirEntries {
-		fileInfo, err := dirEntry.Info()
-		if err != nil {
-			err = walkFunc(name, nil, err)
-			if err != nil {
-				return err
-			}
-		}
-		if err := walkSourceDir(system, name.JoinString(dirEntry.Name()), fileInfo, walkFunc); err != nil {
-			if errors.Is(err, fs.SkipDir) {
-				break
-			}
-			return err
-		}
-	}
-
-	return nil
 }
