@@ -3,11 +3,10 @@ package chezmoi
 import (
 	"fmt"
 	"io/fs"
-	"sort"
 	"testing"
 
+	"github.com/alecthomas/assert/v2"
 	"github.com/muesli/combinator"
-	"github.com/stretchr/testify/require"
 	vfs "github.com/twpayne/go-vfs/v4"
 	"github.com/twpayne/go-vfs/v4/vfst"
 
@@ -17,7 +16,7 @@ import (
 func TestTargetStateEntryApply(t *testing.T) {
 	targetStates := map[string]TargetStateEntry{
 		"dir": &TargetStateDir{
-			perm: 0o777 &^ chezmoitest.Umask,
+			perm: fs.ModePerm &^ chezmoitest.Umask,
 		},
 		"file": &TargetStateFile{
 			perm:         0o666 &^ chezmoitest.Umask,
@@ -28,7 +27,7 @@ func TestTargetStateEntryApply(t *testing.T) {
 			empty: true,
 		},
 		"file_executable": &TargetStateFile{
-			perm:         0o777 &^ chezmoitest.Umask,
+			perm:         fs.ModePerm &^ chezmoitest.Umask,
 			lazyContents: newLazyContents([]byte("#!/bin/sh\n")),
 		},
 		"remove": &TargetStateRemove{},
@@ -37,9 +36,9 @@ func TestTargetStateEntryApply(t *testing.T) {
 		},
 	}
 
-	actualStates := map[string]map[string]interface{}{
+	actualStates := map[string]map[string]any{
 		"dir": {
-			"/home/user/target": &vfst.Dir{Perm: 0o777},
+			"/home/user/target": &vfst.Dir{Perm: fs.ModePerm},
 		},
 		"file": {
 			"/home/user/target": "# contents of file",
@@ -49,15 +48,15 @@ func TestTargetStateEntryApply(t *testing.T) {
 		},
 		"file_executable": {
 			"/home/user/target": &vfst.File{
-				Perm:     0o777,
+				Perm:     fs.ModePerm,
 				Contents: []byte("!/bin/sh\n"),
 			},
 		},
 		"remove": {
-			"/home/user": &vfst.Dir{Perm: 0o777},
+			"/home/user": &vfst.Dir{Perm: fs.ModePerm},
 		},
 		"symlink": {
-			"/home/user": map[string]interface{}{
+			"/home/user": map[string]any{
 				"symlink-target": "",
 				"target":         &vfst.Symlink{Target: "symlink-target"},
 			},
@@ -67,52 +66,53 @@ func TestTargetStateEntryApply(t *testing.T) {
 		},
 	}
 
-	targetStateKeys := make([]string, 0, len(targetStates))
-	for targetStateKey := range targetStates {
-		targetStateKeys = append(targetStateKeys, targetStateKey)
-	}
-	sort.Strings(targetStateKeys)
-
-	actualDestDirStateKeys := make([]string, 0, len(actualStates))
-	for actualDestDirStateKey := range actualStates {
-		actualDestDirStateKeys = append(actualDestDirStateKeys, actualDestDirStateKey)
-	}
-	sort.Strings(actualDestDirStateKeys)
-
 	testData := struct {
 		TargetStateKey        []string
 		ActualDestDirStateKey []string
 	}{
-		TargetStateKey:        targetStateKeys,
-		ActualDestDirStateKey: actualDestDirStateKeys,
+		TargetStateKey:        sortedKeys(targetStates),
+		ActualDestDirStateKey: sortedKeys(actualStates),
 	}
 	var testCases []struct {
 		TargetStateKey        string
 		ActualDestDirStateKey string
 	}
-	require.NoError(t, combinator.Generate(&testCases, testData))
+	assert.NoError(t, combinator.Generate(&testCases, testData))
 
 	for _, tc := range testCases {
-		t.Run(fmt.Sprintf("target_%s_actual_%s", tc.TargetStateKey, tc.ActualDestDirStateKey), func(t *testing.T) {
-			targetState := targetStates[tc.TargetStateKey]
-			actualState := actualStates[tc.ActualDestDirStateKey]
+		t.Run(
+			fmt.Sprintf("target_%s_actual_%s", tc.TargetStateKey, tc.ActualDestDirStateKey),
+			func(t *testing.T) {
+				targetState := targetStates[tc.TargetStateKey]
+				actualState := actualStates[tc.ActualDestDirStateKey]
 
-			chezmoitest.WithTestFS(t, actualState, func(fileSystem vfs.FS) {
-				system := NewRealSystem(fileSystem)
+				chezmoitest.WithTestFS(t, actualState, func(fileSystem vfs.FS) {
+					system := NewRealSystem(fileSystem)
 
-				// Read the initial destination state entry from fileSystem.
-				actualStateEntry, err := NewActualStateEntry(system, NewAbsPath("/home/user/target"), nil, nil)
-				require.NoError(t, err)
+					// Read the initial destination state entry from fileSystem.
+					actualStateEntry, err := NewActualStateEntry(
+						system,
+						NewAbsPath("/home/user/target"),
+						nil,
+						nil,
+					)
+					assert.NoError(t, err)
 
-				// Apply the target state entry.
-				_, err = targetState.Apply(system, nil, actualStateEntry)
-				require.NoError(t, err)
+					// Apply the target state entry.
+					_, err = targetState.Apply(system, nil, actualStateEntry)
+					assert.NoError(t, err)
 
-				// Verify that the actual state entry matches the desired
-				// state.
-				vfst.RunTests(t, fileSystem, "", vfst.TestPath("/home/user/target", targetStateTest(t, targetState)...))
-			})
-		})
+					// Verify that the actual state entry matches the desired
+					// state.
+					vfst.RunTests(
+						t,
+						fileSystem,
+						"",
+						vfst.TestPath("/home/user/target", targetStateTest(t, targetState)...),
+					)
+				})
+			},
+		)
 	}
 }
 
@@ -130,7 +130,7 @@ func targetStateTest(t *testing.T, ts TargetStateEntry) []vfst.PathTest {
 		}
 	case *TargetStateFile:
 		expectedContents, err := ts.Contents()
-		require.NoError(t, err)
+		assert.NoError(t, err)
 		return []vfst.PathTest{
 			vfst.TestModeIsRegular,
 			vfst.TestContents(expectedContents),
@@ -140,7 +140,7 @@ func targetStateTest(t *testing.T, ts TargetStateEntry) []vfst.PathTest {
 		return nil
 	case *TargetStateSymlink:
 		expectedLinkname, err := ts.Linkname()
-		require.NoError(t, err)
+		assert.NoError(t, err)
 		return []vfst.PathTest{
 			vfst.TestModeType(fs.ModeSymlink),
 			vfst.TestSymlinkTarget(expectedLinkname),
