@@ -5,7 +5,7 @@ import (
 	"path/filepath"
 	"testing"
 
-	"github.com/stretchr/testify/require"
+	"github.com/alecthomas/assert/v2"
 	"github.com/twpayne/go-vfs/v4"
 	"github.com/twpayne/go-vfs/v4/vfst"
 
@@ -15,13 +15,13 @@ import (
 func TestApplyCmd(t *testing.T) {
 	for _, tc := range []struct {
 		name      string
-		extraRoot interface{}
+		extraRoot any
 		args      []string
-		tests     []interface{}
+		tests     []any
 	}{
 		{
 			name: "all",
-			tests: []interface{}{
+			tests: []any{
 				vfst.TestPath("/home/user/.create",
 					vfst.TestModeIsRegular,
 					vfst.TestModePerm(0o666&^chezmoitest.Umask),
@@ -29,7 +29,7 @@ func TestApplyCmd(t *testing.T) {
 				),
 				vfst.TestPath("/home/user/.dir",
 					vfst.TestIsDir,
-					vfst.TestModePerm(0o777&^chezmoitest.Umask),
+					vfst.TestModePerm(fs.ModePerm&^chezmoitest.Umask),
 				),
 				vfst.TestPath("/home/user/.dir/file",
 					vfst.TestModeIsRegular,
@@ -38,7 +38,7 @@ func TestApplyCmd(t *testing.T) {
 				),
 				vfst.TestPath("/home/user/.dir/subdir",
 					vfst.TestIsDir,
-					vfst.TestModePerm(0o777&^chezmoitest.Umask),
+					vfst.TestModePerm(fs.ModePerm&^chezmoitest.Umask),
 				),
 				vfst.TestPath("/home/user/.dir/subdir/file",
 					vfst.TestModeIsRegular,
@@ -52,7 +52,7 @@ func TestApplyCmd(t *testing.T) {
 				),
 				vfst.TestPath("/home/user/.executable",
 					vfst.TestModeIsRegular,
-					vfst.TestModePerm(0o777&^chezmoitest.Umask),
+					vfst.TestModePerm(fs.ModePerm&^chezmoitest.Umask),
 					vfst.TestContentsString("# contents of .executable\n"),
 				),
 				vfst.TestPath("/home/user/.file",
@@ -82,7 +82,7 @@ func TestApplyCmd(t *testing.T) {
 		{
 			name: "all_with_--dry-run",
 			args: []string{"--dry-run"},
-			tests: []interface{}{
+			tests: []any{
 				vfst.TestPath("/home/user/.create",
 					vfst.TestDoesNotExist,
 				),
@@ -115,10 +115,10 @@ func TestApplyCmd(t *testing.T) {
 		{
 			name: "dir",
 			args: []string{"~/.dir"},
-			tests: []interface{}{
+			tests: []any{
 				vfst.TestPath("/home/user/.dir",
 					vfst.TestIsDir,
-					vfst.TestModePerm(0o777&^chezmoitest.Umask),
+					vfst.TestModePerm(fs.ModePerm&^chezmoitest.Umask),
 				),
 				vfst.TestPath("/home/user/.dir/file",
 					vfst.TestModeIsRegular,
@@ -127,7 +127,7 @@ func TestApplyCmd(t *testing.T) {
 				),
 				vfst.TestPath("/home/user/.dir/subdir",
 					vfst.TestIsDir,
-					vfst.TestModePerm(0o777&^chezmoitest.Umask),
+					vfst.TestModePerm(fs.ModePerm&^chezmoitest.Umask),
 				),
 				vfst.TestPath("/home/user/.dir/subdir/file",
 					vfst.TestModeIsRegular,
@@ -139,10 +139,10 @@ func TestApplyCmd(t *testing.T) {
 		{
 			name: "dir_with_--recursive=false",
 			args: []string{"~/.dir", "--recursive=false"},
-			tests: []interface{}{
+			tests: []any{
 				vfst.TestPath("/home/user/.dir",
 					vfst.TestIsDir,
-					vfst.TestModePerm(0o777&^chezmoitest.Umask),
+					vfst.TestModePerm(fs.ModePerm&^chezmoitest.Umask),
 				),
 				vfst.TestPath("/home/user/.dir/file",
 					vfst.TestDoesNotExist,
@@ -155,10 +155,10 @@ func TestApplyCmd(t *testing.T) {
 		{
 			name: "create",
 			args: []string{"~/.create"},
-			extraRoot: map[string]interface{}{
+			extraRoot: map[string]any{
 				"/home/user/.create": "# existing contents of .create\n",
 			},
-			tests: []interface{}{
+			tests: []any{
 				vfst.TestPath("/home/user/.create",
 					vfst.TestModeIsRegular,
 					vfst.TestModePerm(0o666&^chezmoitest.Umask),
@@ -168,23 +168,23 @@ func TestApplyCmd(t *testing.T) {
 		},
 	} {
 		t.Run(tc.name, func(t *testing.T) {
-			chezmoitest.WithTestFS(t, map[string]interface{}{
-				"/home/user": map[string]interface{}{
-					".config": map[string]interface{}{
-						"chezmoi": map[string]interface{}{
+			chezmoitest.WithTestFS(t, map[string]any{
+				"/home/user": map[string]any{
+					".config": map[string]any{
+						"chezmoi": map[string]any{
 							"chezmoi.toml": chezmoitest.JoinLines(
 								`[data]`,
 								`  variable = "value"`,
 							),
 						},
 					},
-					".local": map[string]interface{}{
-						"share": map[string]interface{}{
-							"chezmoi": map[string]interface{}{
+					".local": map[string]any{
+						"share": map[string]any{
+							"chezmoi": map[string]any{
 								"create_dot_create": "# contents of .create\n",
-								"dot_dir": map[string]interface{}{
+								"dot_dir": map[string]any{
 									"file": "# contents of .dir/file\n",
-									"subdir": map[string]interface{}{
+									"subdir": map[string]any{
 										"file": "# contents of .dir/subdir/file\n",
 									},
 								},
@@ -203,9 +203,12 @@ func TestApplyCmd(t *testing.T) {
 				},
 			}, func(fileSystem vfs.FS) {
 				if tc.extraRoot != nil {
-					require.NoError(t, vfst.NewBuilder().Build(fileSystem, tc.extraRoot))
+					assert.NoError(t, vfst.NewBuilder().Build(fileSystem, tc.extraRoot))
 				}
-				require.NoError(t, newTestConfig(t, fileSystem).execute(append([]string{"apply"}, tc.args...)))
+				assert.NoError(
+					t,
+					newTestConfig(t, fileSystem).execute(append([]string{"apply"}, tc.args...)),
+				)
 				vfst.RunTests(t, fileSystem, "", tc.tests)
 			})
 		})
