@@ -5,10 +5,10 @@ import (
 	"io/fs"
 	"path/filepath"
 	"runtime"
+	"strconv"
 	"testing"
 
-	"github.com/stretchr/testify/assert"
-	"github.com/stretchr/testify/require"
+	"github.com/alecthomas/assert/v2"
 	vfs "github.com/twpayne/go-vfs/v4"
 	xdg "github.com/twpayne/go-xdg/v6"
 
@@ -28,6 +28,48 @@ func TestAddTemplateFuncPanic(t *testing.T) {
 	})
 }
 
+func TestParseCommand(t *testing.T) {
+	for i, tc := range []struct {
+		command         string
+		args            []string
+		expectedCommand string
+		expectedArgs    []string
+		expectedErr     bool
+	}{
+		{
+			command:         "chezmoi-editor",
+			expectedCommand: "chezmoi-editor",
+		},
+		{
+			command:         `chezmoi-editor -f --nomru -c "au VimLeave * !open -a Terminal"`,
+			expectedCommand: "chezmoi-editor",
+			expectedArgs:    []string{"-f", "--nomru", "-c", "au VimLeave * !open -a Terminal"},
+		},
+		{
+			command:         `"chezmoi editor" $CHEZMOI_TEST_VAR`,
+			args:            []string{"extra-arg"},
+			expectedCommand: "chezmoi editor",
+			expectedArgs:    []string{"chezmoi-test-value", "extra-arg"},
+		},
+		{
+			command:     `"chezmoi editor`,
+			expectedErr: true,
+		},
+	} {
+		t.Run(strconv.Itoa(i), func(t *testing.T) {
+			t.Setenv("CHEZMOI_TEST_VAR", "chezmoi-test-value")
+			actualCommand, actualArgs, err := parseCommand(tc.command, tc.args)
+			if tc.expectedErr {
+				assert.Error(t, err)
+			} else {
+				assert.NoError(t, err)
+				assert.Equal(t, tc.expectedCommand, actualCommand)
+				assert.Equal(t, tc.expectedArgs, actualArgs)
+			}
+		})
+	}
+}
+
 func TestParseConfig(t *testing.T) {
 	for _, tc := range []struct {
 		name          string
@@ -89,11 +131,11 @@ func TestParseConfig(t *testing.T) {
 		},
 	} {
 		t.Run(tc.name, func(t *testing.T) {
-			chezmoitest.WithTestFS(t, map[string]interface{}{
+			chezmoitest.WithTestFS(t, map[string]any{
 				"/home/user/.config/chezmoi/" + tc.filename: tc.contents,
 			}, func(fileSystem vfs.FS) {
 				c := newTestConfig(t, fileSystem)
-				require.NoError(t, c.execute([]string{"init"}))
+				assert.NoError(t, c.execute([]string{"init"}))
 				assert.Equal(t, tc.expectedColor, c.Color.Value(c.colorAutoFunc))
 			})
 		})
@@ -113,58 +155,6 @@ func TestUpperSnakeCaseToCamelCase(t *testing.T) {
 	}
 }
 
-func TestValidateKeys(t *testing.T) {
-	for _, tc := range []struct {
-		data        interface{}
-		expectedErr bool
-	}{
-		{
-			data:        nil,
-			expectedErr: false,
-		},
-		{
-			data: map[string]interface{}{
-				"foo":                    "bar",
-				"a":                      0,
-				"_x9":                    false,
-				"ThisVariableIsExported": nil,
-				"αβ":                     "",
-			},
-			expectedErr: false,
-		},
-		{
-			data: map[string]interface{}{
-				"foo-foo": "bar",
-			},
-			expectedErr: true,
-		},
-		{
-			data: map[string]interface{}{
-				"foo": map[string]interface{}{
-					"bar-bar": "baz",
-				},
-			},
-			expectedErr: true,
-		},
-		{
-			data: map[string]interface{}{
-				"foo": []interface{}{
-					map[string]interface{}{
-						"bar-bar": "baz",
-					},
-				},
-			},
-			expectedErr: true,
-		},
-	} {
-		if tc.expectedErr {
-			assert.Error(t, validateKeys(tc.data, identifierRx))
-		} else {
-			assert.NoError(t, validateKeys(tc.data, identifierRx))
-		}
-	}
-}
-
 func newTestConfig(t *testing.T, fileSystem vfs.FS, options ...configOption) *Config {
 	t.Helper()
 	system := chezmoi.NewRealSystem(fileSystem)
@@ -176,9 +166,12 @@ func newTestConfig(t *testing.T, fileSystem vfs.FS, options ...configOption) *Co
 			withTestFS(fileSystem),
 			withTestUser(t, "user"),
 			withUmask(chezmoitest.Umask),
+			withVersionInfo(VersionInfo{
+				Version: "2.0.0",
+			}),
 		}, options...)...,
 	)
-	require.NoError(t, err)
+	assert.NoError(t, err)
 	return config
 }
 
@@ -196,6 +189,13 @@ func withDestSystem(destSystem chezmoi.System) configOption {
 	}
 }
 
+func withNoTTY(noTTY bool) configOption {
+	return func(c *Config) error {
+		c.noTTY = noTTY
+		return nil
+	}
+}
+
 func withSourceSystem(sourceSystem chezmoi.System) configOption {
 	return func(c *Config) error {
 		c.sourceSystem = sourceSystem
@@ -239,11 +239,11 @@ func withTestUser(t *testing.T, username string) configOption {
 			config.homeDir = filepath.Join("/", "home", username)
 			env = "HOME"
 		}
-		testSetenv(t, env, config.homeDir)
+		t.Setenv(env, config.homeDir)
 		var err error
 		config.homeDirAbsPath, err = chezmoi.NormalizePath(config.homeDir)
 		if err != nil {
-			panic(err)
+			t.Fatal(err)
 		}
 		config.CacheDirAbsPath = config.homeDirAbsPath.JoinString(".cache", "chezmoi")
 		config.SourceDirAbsPath = config.homeDirAbsPath.JoinString(".local", "share", "chezmoi")
