@@ -1,10 +1,14 @@
 package cmd
 
+// FIXME add check for $TMPDIR mount options (specifically noexec)
+
 import (
 	"bytes"
+	"context"
 	"errors"
 	"fmt"
 	"io/fs"
+	"net/http"
 	"os"
 	"os/exec"
 	"regexp"
@@ -12,42 +16,64 @@ import (
 	"sort"
 	"strings"
 	"text/tabwriter"
+	"time"
 
 	"github.com/coreos/go-semver/semver"
+	"github.com/google/go-github/v53/github"
 	"github.com/spf13/cobra"
-	"github.com/spf13/viper"
-	"github.com/twpayne/go-shell"
 	"github.com/twpayne/go-xdg/v6"
 
 	"github.com/twpayne/chezmoi/v2/internal/chezmoi"
 	"github.com/twpayne/chezmoi/v2/internal/chezmoilog"
+	"github.com/twpayne/chezmoi/v2/internal/git"
+	"github.com/twpayne/chezmoi/v2/internal/shell"
 )
 
 // A checkResult is the result of a check.
 type checkResult int
 
 const (
+	checkResultFailed  checkResult = -2 // The check could not be completed.
 	checkResultSkipped checkResult = -1 // The check was skipped.
 	checkResultOK      checkResult = 0  // The check completed and did not find any problems.
 	checkResultInfo    checkResult = 1  // The check completed and found something interesting, but not a problem.
 	checkResultWarning checkResult = 2  // The check completed and found something that might indicate a problem.
 	checkResultError   checkResult = 3  // The check completed and found a definite problem.
-	checkResultFailed  checkResult = 4  // The check could not be completed.
+)
+
+// A gitStatus is the status of a git working copy.
+type gitStatus string
+
+const (
+	gitStatusNotAWorkingCopy gitStatus = ""
+	gitStatusClean           gitStatus = "clean"
+	gitStatusDirty           gitStatus = "dirty"
+	gitStatusError           gitStatus = "error"
 )
 
 // A check is an individual check.
 type check interface {
-	Name() string                                                                    // Name returns the check's name.
-	Run(system chezmoi.System, homeDirAbsPath chezmoi.AbsPath) (checkResult, string) // Run runs the check.
+	Name() string // Name returns the check's name.
+	Run(
+		system chezmoi.System,
+		homeDirAbsPath chezmoi.AbsPath,
+	) (checkResult, string) // Run runs the check.
 }
 
 var checkResultStr = map[checkResult]string{
+	checkResultFailed:  "failed",
 	checkResultSkipped: "skipped",
 	checkResultOK:      "ok",
 	checkResultInfo:    "info",
 	checkResultWarning: "warning",
 	checkResultError:   "error",
-	checkResultFailed:  "failed",
+}
+
+// An argsCheck checks that arguments for a binary.
+type argsCheck struct {
+	name    string
+	command string
+	args    []string
 }
 
 // A binaryCheck checks that a binary called name is installed and optionally at
@@ -90,18 +116,27 @@ type fileCheck struct {
 // A goVersionCheck checks the Go version.
 type goVersionCheck struct{}
 
+// A latestVersionCheck checks the latest version.
+type latestVersionCheck struct {
+	httpClient    *http.Client
+	httpClientErr error
+	owner         string
+	repo          string
+	version       semver.Version
+}
+
 // An osArchCheck checks that runtime.GOOS and runtime.GOARCH are supported.
 type osArchCheck struct{}
 
+// A skippedCheck is a check that is skipped.
+type skippedCheck struct{}
+
 // A suspiciousEntriesCheck checks that a source directory does not contain any
 // suspicious files.
 type suspiciousEntriesCheck struct {
 	dirname chezmoi.AbsPath
 }
 
-// A umaskCheck checks the umask.
-type umaskCheck struct{}
-
 // A upgradeMethodCheck checks the upgrade method.
 type upgradeMethodCheck struct{}
 
@@ -119,10 +154,10 @@ func (c *Config) newDoctorCmd() *cobra.Command {
 		Example: example("doctor"),
 		Long:    mustLongHelp("doctor"),
 		RunE:    c.runDoctorCmd,
-		Annotations: map[string]string{
-			doesNotRequireValidConfig: "true",
-			runsCommands:              "true",
-		},
+		Annotations: newAnnotations(
+			runsCommands,
+			runsWithInvalidConfig,
+		),
 	}
 
 	return doctorCmd
@@ -133,17 +168,29 @@ func (c *Config) runDoctorCmd(cmd *cobra.Command, args []string) error {
 	if err != nil {
 		return err
 	}
-	shell, _ := shell.CurrentUserShell()
-	editCommand, _ := c.editor(nil)
+	httpClient, httpClientErr := c.getHTTPClient()
+	shellCommand, _ := shell.CurrentUserShell()
+	shellCommand, shellArgs, _ := parseCommand(shellCommand, nil)
+	cdCommand, cdArgs, _ := c.cdCommand()
+	editCommand, editArgs, _ := c.editor(nil)
 	checks := []check{
 		&versionCheck{
 			versionInfo: c.versionInfo,
 			versionStr:  c.versionStr,
 		},
-		&osArchCheck{},
-		&goVersionCheck{},
-		&executableCheck{},
-		&upgradeMethodCheck{},
+		&latestVersionCheck{
+			httpClient:    httpClient,
+			httpClientErr: httpClientErr,
+			owner:         gitHubOwner,
+			repo:          gitHubRepo,
+			version:       c.version,
+		},
+		osArchCheck{},
+		unameCheck{},
+		systeminfoCheck{},
+		goVersionCheck{},
+		executableCheck{},
+		upgradeMethodCheck{},
 		&configFileCheck{
 			basename: chezmoiRelPath,
 			bds:      c.bds,
@@ -164,19 +211,35 @@ func (c *Config) runDoctorCmd(cmd *cobra.Command, args []string) error {
 			name:    "dest-dir",
 			dirname: c.DestDirAbsPath,
 		},
+		umaskCheck{},
 		&binaryCheck{
-			name:       "shell",
-			binaryname: shell,
+			name:       "cd-command",
+			binaryname: cdCommand,
 			ifNotSet:   checkResultError,
 			ifNotExist: checkResultError,
 		},
+		&argsCheck{
+			name:    "cd-args",
+			command: cdCommand,
+			args:    cdArgs,
+		},
+		&binaryCheck{
+			name:       "diff-command",
+			binaryname: c.Diff.Command,
+			ifNotSet:   checkResultInfo,
+			ifNotExist: checkResultWarning,
+		},
 		&binaryCheck{
 			name:       "edit-command",
 			binaryname: editCommand,
 			ifNotSet:   checkResultWarning,
 			ifNotExist: checkResultWarning,
 		},
-		&umaskCheck{},
+		&argsCheck{
+			name:    "edit-args",
+			command: editCommand,
+			args:    editArgs,
+		},
 		&binaryCheck{
 			name:        "git-command",
 			binaryname:  c.Git.Command,
@@ -191,6 +254,17 @@ func (c *Config) runDoctorCmd(cmd *cobra.Command, args []string) error {
 			ifNotSet:   checkResultWarning,
 			ifNotExist: checkResultWarning,
 		},
+		&binaryCheck{
+			name:       "shell-command",
+			binaryname: shellCommand,
+			ifNotSet:   checkResultError,
+			ifNotExist: checkResultError,
+		},
+		&argsCheck{
+			name:    "shell-args",
+			command: shellCommand,
+			args:    shellArgs,
+		},
 		&binaryCheck{
 			name:        "age-command",
 			binaryname:  c.Age.Command,
@@ -221,7 +295,7 @@ func (c *Config) runDoctorCmd(cmd *cobra.Command, args []string) error {
 			ifNotSet:    checkResultWarning,
 			ifNotExist:  checkResultInfo,
 			versionArgs: []string{"--version"},
-			versionRx:   regexp.MustCompile(`^(\d+\.\d+\.\d+)`),
+			versionRx:   onepasswordVersionRx,
 		},
 		&binaryCheck{
 			name:        "bitwarden-command",
@@ -231,6 +305,14 @@ func (c *Config) runDoctorCmd(cmd *cobra.Command, args []string) error {
 			versionArgs: []string{"--version"},
 			versionRx:   regexp.MustCompile(`^(\d+\.\d+\.\d+)`),
 		},
+		&binaryCheck{
+			name:        "dashlane-command",
+			binaryname:  c.Dashlane.Command,
+			ifNotSet:    checkResultWarning,
+			ifNotExist:  checkResultInfo,
+			versionArgs: []string{"--version"},
+			versionRx:   regexp.MustCompile(`^(\d+\.\d+\.\d+)`),
+		},
 		&binaryCheck{
 			name:        "gopass-command",
 			binaryname:  c.Gopass.Command,
@@ -254,6 +336,14 @@ func (c *Config) runDoctorCmd(cmd *cobra.Command, args []string) error {
 			ifNotSet:   checkResultInfo,
 			ifNotExist: checkResultInfo,
 		},
+		&binaryCheck{
+			name:        "keeper-command",
+			binaryname:  c.Keeper.Command,
+			ifNotSet:    checkResultWarning,
+			ifNotExist:  checkResultInfo,
+			versionArgs: []string{"version"},
+			versionRx:   regexp.MustCompile(`^Commander\s+Version:\s+(\d+\.\d+\.\d+)`),
+		},
 		&binaryCheck{
 			name:        "lastpass-command",
 			binaryname:  c.Lastpass.Command,
@@ -271,6 +361,24 @@ func (c *Config) runDoctorCmd(cmd *cobra.Command, args []string) error {
 			versionArgs: []string{"version"},
 			versionRx:   regexp.MustCompile(`(?m)=\s*v(\d+\.\d+\.\d+)`),
 		},
+		&binaryCheck{
+			name:        "passhole-command",
+			binaryname:  c.Passhole.Command,
+			ifNotSet:    checkResultWarning,
+			ifNotExist:  checkResultInfo,
+			versionArgs: []string{"--version"},
+			versionRx:   regexp.MustCompile(`^(\d+\.\d+\.\d+)`),
+			minVersion:  &passholeMinVersion,
+		},
+		&binaryCheck{
+			name:        "rbw-command",
+			binaryname:  c.RBW.Command,
+			ifNotSet:    checkResultWarning,
+			ifNotExist:  checkResultInfo,
+			versionArgs: []string{"--version"},
+			versionRx:   regexp.MustCompile(`^rbw\s+(\d+\.\d+\.\d+)`),
+			minVersion:  &rbwMinVersion,
+		},
 		&binaryCheck{
 			name:        "vault-command",
 			binaryname:  c.Vault.Command,
@@ -279,6 +387,15 @@ func (c *Config) runDoctorCmd(cmd *cobra.Command, args []string) error {
 			versionArgs: []string{"version"},
 			versionRx:   regexp.MustCompile(`^Vault\s+v(\d+\.\d+\.\d+)`),
 		},
+		&binaryCheck{
+			name:        "vlt-command",
+			binaryname:  c.HCPVaultSecrets.Command,
+			ifNotSet:    checkResultWarning,
+			ifNotExist:  checkResultInfo,
+			versionArgs: []string{"version"},
+			versionRx:   regexp.MustCompile(`^(\d+\.\d+\.\d+)`),
+			minVersion:  &vltMinVersion,
+		},
 		&binaryCheck{
 			name:       "secret-command",
 			binaryname: c.Secret.Command,
@@ -299,7 +416,13 @@ func (c *Config) runDoctorCmd(cmd *cobra.Command, args []string) error {
 		// output of chezmoi doctor is often posted publicly and would otherwise
 		// reveal the user's username.
 		message = strings.ReplaceAll(message, homeDirAbsPath.String(), "~")
-		fmt.Fprintf(resultWriter, "%s\t%s\t%s\n", checkResultStr[checkResult], check.Name(), message)
+		fmt.Fprintf(
+			resultWriter,
+			"%s\t%s\t%s\n",
+			checkResultStr[checkResult],
+			check.Name(),
+			message,
+		)
 		if checkResult > worstResult {
 			worstResult = checkResult
 		}
@@ -313,17 +436,31 @@ func (c *Config) runDoctorCmd(cmd *cobra.Command, args []string) error {
 	return nil
 }
 
+func (c *argsCheck) Name() string {
+	return c.name
+}
+
+func (c *argsCheck) Run(
+	system chezmoi.System,
+	homeDirAbsPath chezmoi.AbsPath,
+) (checkResult, string) {
+	return checkResultOK, shellQuoteCommand(c.command, c.args)
+}
+
 func (c *binaryCheck) Name() string {
 	return c.name
 }
 
-func (c *binaryCheck) Run(system chezmoi.System, homeDirAbsPath chezmoi.AbsPath) (checkResult, string) {
+func (c *binaryCheck) Run(
+	system chezmoi.System,
+	homeDirAbsPath chezmoi.AbsPath,
+) (checkResult, string) {
 	if c.binaryname == "" {
 		return c.ifNotSet, "not set"
 	}
 
 	var pathAbsPath chezmoi.AbsPath
-	switch path, err := exec.LookPath(c.binaryname); {
+	switch path, err := chezmoi.LookPath(c.binaryname); {
 	case errors.Is(err, exec.ErrNotFound):
 		return c.ifNotExist, fmt.Sprintf("%s not found in $PATH", c.binaryname)
 	case err != nil:
@@ -339,8 +476,7 @@ func (c *binaryCheck) Run(system chezmoi.System, homeDirAbsPath chezmoi.AbsPath)
 		return checkResultOK, fmt.Sprintf("found %s", pathAbsPath)
 	}
 
-	//nolint:gosec
-	cmd := exec.Command(pathAbsPath.String(), c.versionArgs...)
+	cmd := exec.Command(pathAbsPath.String(), c.versionArgs...) //nolint:gosec
 	output, err := chezmoilog.LogCmdCombinedOutput(cmd)
 	if err != nil {
 		return checkResultFailed, err.Error()
@@ -350,7 +486,11 @@ func (c *binaryCheck) Run(system chezmoi.System, homeDirAbsPath chezmoi.AbsPath)
 	if c.versionRx != nil {
 		match := c.versionRx.FindSubmatch(versionBytes)
 		if len(match) != 2 {
-			s := fmt.Sprintf("found %s, cannot parse version from %s", pathAbsPath, bytes.TrimSpace(versionBytes))
+			s := fmt.Sprintf(
+				"found %s, cannot parse version from %s",
+				pathAbsPath,
+				bytes.TrimSpace(versionBytes),
+			)
 			return checkResultWarning, s
 		}
 		versionBytes = match[1]
@@ -361,7 +501,12 @@ func (c *binaryCheck) Run(system chezmoi.System, homeDirAbsPath chezmoi.AbsPath)
 	}
 
 	if c.minVersion != nil && version.LessThan(*c.minVersion) {
-		return checkResultError, fmt.Sprintf("found %s, version %s, need %s", pathAbsPath, version, c.minVersion)
+		return checkResultError, fmt.Sprintf(
+			"found %s, version %s, need %s",
+			pathAbsPath,
+			version,
+			c.minVersion,
+		)
 	}
 
 	return checkResultOK, fmt.Sprintf("found %s, version %s", pathAbsPath, version)
@@ -371,15 +516,21 @@ func (c *configFileCheck) Name() string {
 	return "config-file"
 }
 
-func (c *configFileCheck) Run(system chezmoi.System, homeDirAbsPath chezmoi.AbsPath) (checkResult, string) {
+func (c *configFileCheck) Run(
+	system chezmoi.System,
+	homeDirAbsPath chezmoi.AbsPath,
+) (checkResult, string) {
 	filenameAbsPaths := make(map[chezmoi.AbsPath]struct{})
 	for _, dir := range append([]string{c.bds.ConfigHome}, c.bds.ConfigDirs...) {
 		configDirAbsPath, err := chezmoi.NewAbsPathFromExtPath(dir, homeDirAbsPath)
 		if err != nil {
 			return checkResultFailed, err.Error()
 		}
-		for _, extension := range viper.SupportedExts {
-			filenameAbsPath := configDirAbsPath.Join(c.basename, chezmoi.NewRelPath(c.basename.String()+"."+extension))
+		for _, extension := range chezmoi.FormatExtensions {
+			filenameAbsPath := configDirAbsPath.Join(
+				c.basename,
+				chezmoi.NewRelPath(c.basename.String()+"."+extension),
+			)
 			if _, err := system.Stat(filenameAbsPath); err == nil {
 				filenameAbsPaths[filenameAbsPath] = struct{}{}
 			}
@@ -389,23 +540,41 @@ func (c *configFileCheck) Run(system chezmoi.System, homeDirAbsPath chezmoi.AbsP
 	case 0:
 		return checkResultOK, "no config file found"
 	case 1:
-		var filenameAbsPath chezmoi.AbsPath
-		for filenameAbsPath = range filenameAbsPaths {
-		}
+		filenameAbsPath := anyKey(filenameAbsPaths)
 		if filenameAbsPath != c.expected {
-			return checkResultFailed, fmt.Sprintf("found %s, expected %s", filenameAbsPath, c.expected)
+			return checkResultFailed, fmt.Sprintf(
+				"found %s, expected %s",
+				filenameAbsPath,
+				c.expected,
+			)
 		}
-		if _, err := system.ReadFile(filenameAbsPath); err != nil {
+		config, err := newConfig()
+		if err != nil {
+			return checkResultError, err.Error()
+		}
+		if err := config.decodeConfigFile(filenameAbsPath, &config.ConfigFile); err != nil {
 			return checkResultError, fmt.Sprintf("%s: %v", filenameAbsPath, err)
 		}
-		return checkResultOK, filenameAbsPath.String()
+		fileInfo, err := system.Stat(filenameAbsPath)
+		if err != nil {
+			return checkResultError, fmt.Sprintf("%s: %v", filenameAbsPath, err)
+		}
+		message := fmt.Sprintf(
+			"%s, last modified %s",
+			filenameAbsPath.String(),
+			fileInfo.ModTime().Format(time.RFC3339),
+		)
+		return checkResultOK, message
 	default:
 		filenameStrs := make([]string, 0, len(filenameAbsPaths))
 		for filenameAbsPath := range filenameAbsPaths {
 			filenameStrs = append(filenameStrs, filenameAbsPath.String())
 		}
 		sort.Strings(filenameStrs)
-		return checkResultWarning, fmt.Sprintf("%s: multiple config files", englishList(filenameStrs))
+		return checkResultWarning, fmt.Sprintf(
+			"%s: multiple config files",
+			englishList(filenameStrs),
+		)
 	}
 }
 
@@ -413,18 +582,64 @@ func (c *dirCheck) Name() string {
 	return c.name
 }
 
-func (c *dirCheck) Run(system chezmoi.System, homeDirAbsPath chezmoi.AbsPath) (checkResult, string) {
-	if _, err := system.ReadDir(c.dirname); err != nil {
+func (c *dirCheck) Run(
+	system chezmoi.System,
+	homeDirAbsPath chezmoi.AbsPath,
+) (checkResult, string) {
+	dirEntries, err := system.ReadDir(c.dirname)
+	if err != nil {
 		return checkResultError, err.Error()
 	}
-	return checkResultOK, fmt.Sprintf("%s is a directory", c.dirname)
+
+	gitStatus := gitStatusNotAWorkingCopy
+	for _, dirEntry := range dirEntries {
+		if dirEntry.Name() != ".git" {
+			continue
+		}
+		cmd := exec.Command( //nolint:gosec
+			"git",
+			"-C",
+			c.dirname.String(),
+			"status",
+			"--porcelain=v2",
+		)
+		output, err := cmd.Output()
+		if err != nil {
+			gitStatus = gitStatusError
+			break
+		}
+		switch status, err := git.ParseStatusPorcelainV2(output); {
+		case err != nil:
+			gitStatus = gitStatusError
+		case status.Empty():
+			gitStatus = gitStatusClean
+		default:
+			gitStatus = gitStatusDirty
+		}
+		break
+	}
+	switch gitStatus {
+	case gitStatusNotAWorkingCopy:
+		return checkResultOK, fmt.Sprintf("%s is a directory", c.dirname)
+	case gitStatusClean:
+		return checkResultOK, fmt.Sprintf("%s is a git working tree (clean)", c.dirname)
+	case gitStatusDirty:
+		return checkResultWarning, fmt.Sprintf("%s is a git working tree (dirty)", c.dirname)
+	case gitStatusError:
+		return checkResultError, fmt.Sprintf("%s is a git working tree (error)", c.dirname)
+	default:
+		panic(fmt.Sprintf("%s: unknown git status", gitStatus))
+	}
 }
 
-func (c *executableCheck) Name() string {
+func (executableCheck) Name() string {
 	return "executable"
 }
 
-func (c *executableCheck) Run(system chezmoi.System, homeDirAbsPath chezmoi.AbsPath) (checkResult, string) {
+func (executableCheck) Run(
+	system chezmoi.System,
+	homeDirAbsPath chezmoi.AbsPath,
+) (checkResult, string) {
 	executable, err := os.Executable()
 	if err != nil {
 		return checkResultError, err.Error()
@@ -440,7 +655,10 @@ func (c *fileCheck) Name() string {
 	return c.name
 }
 
-func (c *fileCheck) Run(system chezmoi.System, homeDirAbsPath chezmoi.AbsPath) (checkResult, string) {
+func (c *fileCheck) Run(
+	system chezmoi.System,
+	homeDirAbsPath chezmoi.AbsPath,
+) (checkResult, string) {
 	if c.filename.Empty() {
 		return c.ifNotSet, "not set"
 	}
@@ -459,17 +677,64 @@ func (goVersionCheck) Name() string {
 	return "go-version"
 }
 
-func (goVersionCheck) Run(system chezmoi.System, homeDirAbsPath chezmoi.AbsPath) (checkResult, string) {
+func (goVersionCheck) Run(
+	system chezmoi.System,
+	homeDirAbsPath chezmoi.AbsPath,
+) (checkResult, string) {
 	return checkResultOK, fmt.Sprintf("%s (%s)", runtime.Version(), runtime.Compiler)
 }
 
+func (c *latestVersionCheck) Name() string {
+	return "latest-version"
+}
+
+func (c *latestVersionCheck) Run(
+	system chezmoi.System,
+	homeDirAbsPath chezmoi.AbsPath,
+) (checkResult, string) {
+	if c.httpClientErr != nil {
+		return checkResultFailed, c.httpClientErr.Error()
+	}
+
+	ctx := context.Background()
+
+	gitHubClient := chezmoi.NewGitHubClient(ctx, c.httpClient)
+	rr, _, err := gitHubClient.Repositories.GetLatestRelease(ctx, c.owner, c.repo)
+	var rateLimitErr *github.RateLimitError
+	var abuseRateLimitErr *github.AbuseRateLimitError
+	switch {
+	case err == nil:
+		// Do nothing.
+	case errors.As(err, &rateLimitErr):
+		return checkResultFailed, "GitHub rate limit exceeded"
+	case errors.As(err, &abuseRateLimitErr):
+		return checkResultFailed, "GitHub abuse rate limit exceeded"
+	default:
+		return checkResultFailed, err.Error()
+	}
+
+	version, err := semver.NewVersion(strings.TrimPrefix(rr.GetName(), "v"))
+	if err != nil {
+		return checkResultError, err.Error()
+	}
+
+	checkResult := checkResultOK
+	if c.version.LessThan(*version) {
+		checkResult = checkResultWarning
+	}
+	return checkResult, "v" + version.String()
+}
+
 func (osArchCheck) Name() string {
 	return "os-arch"
 }
 
-func (osArchCheck) Run(system chezmoi.System, homeDirAbsPath chezmoi.AbsPath) (checkResult, string) {
+func (osArchCheck) Run(
+	system chezmoi.System,
+	homeDirAbsPath chezmoi.AbsPath,
+) (checkResult, string) {
 	fields := []string{runtime.GOOS + "/" + runtime.GOARCH}
-	if osRelease, err := chezmoi.OSRelease(system); err == nil {
+	if osRelease, err := chezmoi.OSRelease(system.UnderlyingFS()); err == nil {
 		if name, ok := osRelease["NAME"].(string); ok {
 			if version, ok := osRelease["VERSION"].(string); ok {
 				fields = append(fields, "("+name+" "+version+")")
@@ -481,18 +746,37 @@ func (osArchCheck) Run(system chezmoi.System, homeDirAbsPath chezmoi.AbsPath) (c
 	return checkResultOK, strings.Join(fields, " ")
 }
 
+func (skippedCheck) Name() string {
+	return "skipped"
+}
+
+func (skippedCheck) Run(
+	system chezmoi.System,
+	homeDirAbsPath chezmoi.AbsPath,
+) (checkResult, string) {
+	return checkResultSkipped, ""
+}
+
 func (c *suspiciousEntriesCheck) Name() string {
 	return "suspicious-entries"
 }
 
-func (c *suspiciousEntriesCheck) Run(system chezmoi.System, homeDirAbsPath chezmoi.AbsPath) (checkResult, string) {
+func (c *suspiciousEntriesCheck) Run(
+	system chezmoi.System,
+	homeDirAbsPath chezmoi.AbsPath,
+) (checkResult, string) {
+	// FIXME include user-defined suffixes from age and gpg configs
+	encryptedSuffixes := []string{
+		defaultAgeEncryptionConfig.Suffix,
+		defaultGPGEncryptionConfig.Suffix,
+	}
 	// FIXME check that config file templates are in root
 	var suspiciousEntries []string
 	walkFunc := func(absPath chezmoi.AbsPath, fileInfo fs.FileInfo, err error) error {
 		if err != nil {
 			return err
 		}
-		if chezmoi.SuspiciousSourceDirEntry(absPath.Base(), fileInfo) {
+		if chezmoi.SuspiciousSourceDirEntry(absPath.Base(), fileInfo, encryptedSuffixes) {
 			suspiciousEntries = append(suspiciousEntries, absPath.String())
 		}
 		return nil
@@ -509,11 +793,14 @@ func (c *suspiciousEntriesCheck) Run(system chezmoi.System, homeDirAbsPath chezm
 	return checkResultOK, "no suspicious entries"
 }
 
-func (c *umaskCheck) Name() string {
-	return "umask"
+func (upgradeMethodCheck) Name() string {
+	return "upgrade-method"
 }
 
-func (c *upgradeMethodCheck) Run(system chezmoi.System, homeDirAbsPath chezmoi.AbsPath) (checkResult, string) {
+func (upgradeMethodCheck) Run(
+	system chezmoi.System,
+	homeDirAbsPath chezmoi.AbsPath,
+) (checkResult, string) {
 	executable, err := os.Executable()
 	if err != nil {
 		return checkResultFailed, err.Error()
@@ -528,17 +815,26 @@ func (c *upgradeMethodCheck) Run(system chezmoi.System, homeDirAbsPath chezmoi.A
 	return checkResultOK, method
 }
 
-func (c *upgradeMethodCheck) Name() string {
-	return "upgrade-method"
-}
-
 func (c *versionCheck) Name() string {
 	return "version"
 }
 
-func (c *versionCheck) Run(system chezmoi.System, homeDirAbsPath chezmoi.AbsPath) (checkResult, string) {
+func (c *versionCheck) Run(
+	system chezmoi.System,
+	homeDirAbsPath chezmoi.AbsPath,
+) (checkResult, string) {
 	if c.versionInfo.Version == "" || c.versionInfo.Commit == "" {
 		return checkResultWarning, c.versionStr
 	}
 	return checkResultOK, c.versionStr
 }
+
+// anyKey returns any key from m. It is typically used to return a single key
+// when m is known to contain exactly one element.
+func anyKey[M ~map[K]V, K comparable, V any](m M) K { //nolint:ireturn
+	for k := range m {
+		return k
+	}
+	var k K
+	return k
+}
