@@ -7,9 +7,7 @@ import (
 )
 
 type dumpCmdConfig struct {
-	exclude   *chezmoi.EntryTypeSet
-	format    writeDataFormat
-	include   *chezmoi.EntryTypeSet
+	filter    *chezmoi.EntryTypeFilter
 	init      bool
 	recursive bool
 }
@@ -22,17 +20,29 @@ func (c *Config) newDumpCmd() *cobra.Command {
 		Example:           example("dump"),
 		ValidArgsFunction: c.targetValidArgs,
 		RunE:              c.runDumpCmd,
-		Annotations: map[string]string{
-			persistentStateMode: persistentStateModeEmpty,
-		},
+		Annotations: newAnnotations(
+			persistentStateModeReadMockWrite,
+			requiresSourceDirectory,
+		),
 	}
 
 	flags := dumpCmd.Flags()
-	flags.VarP(c.dump.exclude, "exclude", "x", "Exclude entry types")
-	flags.VarP(&c.dump.format, "format", "f", "Set output format")
-	flags.VarP(c.dump.include, "include", "i", "Include entry types")
-	flags.BoolVar(&c.dump.init, "init", c.update.init, "Recreate config file from template")
-	flags.BoolVarP(&c.dump.recursive, "recursive", "r", c.dump.recursive, "Recurse into subdirectories")
+	flags.VarP(c.dump.filter.Exclude, "exclude", "x", "Exclude entry types")
+	flags.VarP(&c.Format, "format", "f", "Output format")
+	flags.VarP(c.dump.filter.Include, "include", "i", "Include entry types")
+	flags.BoolVar(&c.dump.init, "init", c.dump.init, "Recreate config file from template")
+	flags.BoolVarP(
+		&c.dump.recursive,
+		"recursive",
+		"r",
+		c.dump.recursive,
+		"Recurse into subdirectories",
+	)
+	if err := dumpCmd.RegisterFlagCompletionFunc("format", writeDataFormatFlagCompletionFunc); err != nil {
+		panic(err)
+	}
+
+	registerExcludeIncludeFlagCompletionFuncs(dumpCmd)
 
 	return dumpCmd
 }
@@ -40,12 +50,13 @@ func (c *Config) newDumpCmd() *cobra.Command {
 func (c *Config) runDumpCmd(cmd *cobra.Command, args []string) error {
 	dumpSystem := chezmoi.NewDumpSystem()
 	if err := c.applyArgs(cmd.Context(), dumpSystem, chezmoi.EmptyAbsPath, args, applyArgsOptions{
-		include:   c.dump.include.Sub(c.dump.exclude),
+		cmd:       cmd,
+		filter:    c.dump.filter,
 		init:      c.dump.init,
 		recursive: c.dump.recursive,
 		umask:     c.Umask,
 	}); err != nil {
 		return err
 	}
-	return c.marshal(c.dump.format, dumpSystem.Data())
+	return c.marshal(c.Format, dumpSystem.Data())
 }
