@@ -5,19 +5,20 @@ import (
 	"runtime"
 	"time"
 
+	"github.com/fsnotify/fsnotify"
 	"github.com/spf13/cobra"
 
 	"github.com/twpayne/chezmoi/v2/internal/chezmoi"
 )
 
 type editCmdConfig struct {
-	Command     string        `mapstructure:"command"`
-	Args        []string      `mapstructure:"args"`
-	Hardlink    bool          `mapstructure:"hardlink"`
-	MinDuration time.Duration `mapstructure:"minDuration"`
-	apply       bool
-	exclude     *chezmoi.EntryTypeSet
-	include     *chezmoi.EntryTypeSet
+	Command     string        `json:"command"     mapstructure:"command"     yaml:"command"`
+	Args        []string      `json:"args"        mapstructure:"args"        yaml:"args"`
+	Hardlink    bool          `json:"hardlink"    mapstructure:"hardlink"    yaml:"hardlink"`
+	MinDuration time.Duration `json:"minDuration" mapstructure:"minDuration" yaml:"minDuration"`
+	Watch       bool          `json:"watch"       mapstructure:"watch"       yaml:"watch"`
+	Apply       bool          `json:"apply"       mapstructure:"apply"       yaml:"apply"`
+	filter      *chezmoi.EntryTypeFilter
 	init        bool
 }
 
@@ -28,34 +29,43 @@ func (c *Config) newEditCmd() *cobra.Command {
 		Long:              mustLongHelp("edit"),
 		Example:           example("edit"),
 		ValidArgsFunction: c.targetValidArgs,
-		RunE:              c.makeRunEWithSourceState(c.runEditCmd),
-		Annotations: map[string]string{
-			modifiesDestinationDirectory: "true",
-			modifiesSourceDirectory:      "true",
-			persistentStateMode:          persistentStateModeReadWrite,
-			requiresSourceDirectory:      "true",
-			runsCommands:                 "true",
-		},
+		RunE:              c.runEditCmd,
+		Annotations: newAnnotations(
+			modifiesDestinationDirectory,
+			modifiesSourceDirectory,
+			persistentStateModeReadWrite,
+			requiresSourceDirectory,
+			runsCommands,
+		),
 	}
 
 	flags := editCmd.Flags()
-	flags.BoolVarP(&c.Edit.apply, "apply", "a", c.Edit.apply, "Apply after editing")
-	flags.VarP(c.Edit.exclude, "exclude", "x", "Exclude entry types")
-	flags.BoolVar(&c.Edit.Hardlink, "hardlink", c.Edit.Hardlink, "Invoke editor with a hardlink to the source file")
-	flags.VarP(c.Edit.include, "include", "i", "Include entry types")
-	flags.BoolVar(&c.Edit.init, "init", c.update.init, "Recreate config file from template")
+	flags.BoolVarP(&c.Edit.Apply, "apply", "a", c.Edit.Apply, "Apply after editing")
+	flags.VarP(c.Edit.filter.Exclude, "exclude", "x", "Exclude entry types")
+	flags.BoolVar(
+		&c.Edit.Hardlink,
+		"hardlink",
+		c.Edit.Hardlink,
+		"Invoke editor with a hardlink to the source file",
+	)
+	flags.VarP(c.Edit.filter.Include, "include", "i", "Include entry types")
+	flags.BoolVar(&c.Edit.init, "init", c.Edit.init, "Recreate config file from template")
+	flags.BoolVar(&c.Edit.Watch, "watch", c.Edit.Watch, "Apply on save")
+
+	registerExcludeIncludeFlagCompletionFuncs(editCmd)
 
 	return editCmd
 }
 
-func (c *Config) runEditCmd(cmd *cobra.Command, args []string, sourceState *chezmoi.SourceState) error {
+func (c *Config) runEditCmd(cmd *cobra.Command, args []string) error {
 	if len(args) == 0 {
 		if err := c.runEditor([]string{c.WorkingTreeAbsPath.String()}); err != nil {
 			return err
 		}
-		if c.Edit.apply {
+		if c.Edit.Apply {
 			if err := c.applyArgs(cmd.Context(), c.destSystem, c.DestDirAbsPath, noArgs, applyArgsOptions{
-				include:      c.Edit.include.Sub(c.Edit.exclude),
+				cmd:          cmd,
+				filter:       c.Edit.filter,
 				init:         c.Edit.init,
 				recursive:    true,
 				umask:        c.Umask,
@@ -67,8 +77,14 @@ func (c *Config) runEditCmd(cmd *cobra.Command, args []string, sourceState *chez
 		return nil
 	}
 
+	sourceState, err := c.newSourceState(cmd.Context(), cmd)
+	if err != nil {
+		return err
+	}
+
 	targetRelPaths, err := c.targetRelPaths(sourceState, args, targetRelPathsOptions{
 		mustBeInSourceState: true,
+		mustBeManaged:       true,
 	})
 	if err != nil {
 		return err
@@ -80,7 +96,7 @@ func (c *Config) runEditCmd(cmd *cobra.Command, args []string, sourceState *chez
 		decryptedAbsPath chezmoi.AbsPath
 	}
 	var transparentlyDecryptedFiles []transparentlyDecryptedFile
-TARGETRELPATH:
+TARGET_REL_PATH:
 	for _, targetRelPath := range targetRelPaths {
 		sourceStateEntry := sourceState.MustEntry(targetRelPath)
 		sourceRelPath := sourceStateEntry.SourceRelPath()
@@ -144,7 +160,7 @@ TARGETRELPATH:
 			}
 			if err := c.baseSystem.Link(c.SourceDirAbsPath.Join(sourceRelPath.RelPath()), hardlinkAbsPath); err == nil {
 				editorArgs = append(editorArgs, hardlinkAbsPath.String())
-				continue TARGETRELPATH
+				continue TARGET_REL_PATH
 			}
 
 			// Otherwise, fall through to the default option of editing the
@@ -156,31 +172,83 @@ TARGETRELPATH:
 		}
 	}
 
+	postEditFunc := func() error {
+		for _, transparentlyDecryptedFile := range transparentlyDecryptedFiles {
+			contents, err := c.encryption.EncryptFile(transparentlyDecryptedFile.decryptedAbsPath)
+			if err != nil {
+				return err
+			}
+			if err := c.baseSystem.WriteFile(transparentlyDecryptedFile.sourceAbsPath, contents, 0o666); err != nil {
+				return err
+			}
+		}
+
+		if c.Edit.Apply || c.Edit.Watch {
+			// Reset the cached source state to ensure that we re-read any
+			// changed files.
+			//
+			// FIXME Be more precise in what we invalidate. Only the changed
+			// files need to be re-read, not the entire source state.
+			c.resetSourceState()
+
+			if err := c.applyArgs(cmd.Context(), c.destSystem, c.DestDirAbsPath, args, applyArgsOptions{
+				cmd:          cmd,
+				filter:       c.Edit.filter,
+				init:         c.Edit.init,
+				recursive:    true,
+				umask:        c.Umask,
+				preApplyFunc: c.defaultPreApplyFunc,
+			}); err != nil {
+				return err
+			}
+		}
+
+		return nil
+	}
+
+	if c.Edit.Watch {
+		watcher, err := fsnotify.NewWatcher()
+		if err != nil {
+			return err
+		}
+		defer watcher.Close()
+
+		for _, editorArg := range editorArgs {
+			// FIXME watch directories recursively
+			if err := watcher.Add(editorArg); err != nil {
+				return err
+			}
+		}
+
+		go func() {
+			for {
+				select {
+				case event, ok := <-watcher.Events:
+					if !ok {
+						return
+					}
+					c.logger.Debug().
+						Stringer("Op", event.Op).
+						Str("Name", event.Name).
+						Msg("watcher.Events")
+					err := postEditFunc()
+					c.logger.Err(err).
+						Msg("postEditFunc")
+				case _, ok := <-watcher.Errors:
+					if !ok {
+						return
+					}
+					c.logger.Error().
+						Err(err).
+						Msg("watcher.Errors")
+				}
+			}
+		}()
+	}
+
 	if err := c.runEditor(editorArgs); err != nil {
 		return err
 	}
 
-	for _, transparentlyDecryptedFile := range transparentlyDecryptedFiles {
-		contents, err := c.encryption.EncryptFile(transparentlyDecryptedFile.decryptedAbsPath)
-		if err != nil {
-			return err
-		}
-		if err := c.baseSystem.WriteFile(transparentlyDecryptedFile.sourceAbsPath, contents, 0o666); err != nil {
-			return err
-		}
-	}
-
-	if c.Edit.apply {
-		if err := c.applyArgs(cmd.Context(), c.destSystem, c.DestDirAbsPath, args, applyArgsOptions{
-			include:      c.Edit.include,
-			init:         c.Edit.init,
-			recursive:    false,
-			umask:        c.Umask,
-			preApplyFunc: c.defaultPreApplyFunc,
-		}); err != nil {
-			return err
-		}
-	}
-
-	return nil
+	return postEditFunc()
 }
