@@ -5,12 +5,10 @@ import (
 	"bytes"
 	"errors"
 	"fmt"
-	"go/build"
 	"io/fs"
 	"net/http"
 	"net/http/httptest"
 	"os"
-	"os/exec"
 	"path"
 	"path/filepath"
 	"regexp"
@@ -25,12 +23,14 @@ import (
 	"github.com/twpayne/go-vfs/v4"
 	"github.com/twpayne/go-vfs/v4/vfst"
 
+	"github.com/twpayne/chezmoi/v2/internal/chezmoi"
 	"github.com/twpayne/chezmoi/v2/internal/chezmoitest"
 	"github.com/twpayne/chezmoi/v2/internal/cmd"
 )
 
 var (
 	envConditionRx   = regexp.MustCompile(`\Aenv:(\w+)\z`)
+	envVarRx         = regexp.MustCompile(`\$\w+`)
 	umaskConditionRx = regexp.MustCompile(`\Aumask:([0-7]{3})\z`)
 )
 
@@ -55,8 +55,11 @@ func TestScript(t *testing.T) {
 			"chhome":         cmdChHome,
 			"cmpmod":         cmdCmpMod,
 			"edit":           cmdEdit,
+			"expandenv":      cmdExpandEnv,
 			"httpd":          cmdHTTPD,
+			"isdir":          cmdIsDir,
 			"issymlink":      cmdIsSymlink,
+			"lexists":        cmdLExists,
 			"mkfile":         cmdMkFile,
 			"mkageconfig":    cmdMkAgeConfig,
 			"mkgitconfig":    cmdMkGitConfig,
@@ -70,17 +73,6 @@ func TestScript(t *testing.T) {
 			"unix2dos":       cmdUNIX2DOS,
 		},
 		Condition: func(cond string) (bool, error) {
-			// FIXME remove the following statement when
-			// https://github.com/rogpeppe/go-internal/pull/150 is released.
-			if strings.HasPrefix(cond, "build:") {
-				tag := strings.TrimPrefix(cond, "build:")
-				for _, releaseTag := range build.Default.ReleaseTags {
-					if releaseTag == tag {
-						return true, nil
-					}
-				}
-				return false, nil
-			}
 			if result, valid := goosCondition(cond); valid {
 				return result, nil
 			}
@@ -93,7 +85,9 @@ func TestScript(t *testing.T) {
 			}
 			return false, fmt.Errorf("%s: unknown condition", cond)
 		},
-		Setup: setup,
+		RequireExplicitExec: true,
+		RequireUniqueNames:  true,
+		Setup:               setup,
 	})
 }
 
@@ -127,7 +121,7 @@ func cmdChHome(ts *testscript.TestScript, neg bool, args []string) {
 		chezmoiConfigDir = path.Join(homeDir, ".config", "chezmoi")
 		chezmoiSourceDir = path.Join(homeDir, ".local", "share", "chezmoi")
 	)
-	ts.Check(os.MkdirAll(homeDir, 0o777))
+	ts.Check(os.MkdirAll(homeDir, fs.ModePerm))
 	ts.Setenv("HOME", homeDir)
 	ts.Setenv("CHEZMOICONFIGDIR", chezmoiConfigDir)
 	ts.Setenv("CHEZMOISOURCEDIR", chezmoiSourceDir)
@@ -157,7 +151,12 @@ func cmdCmpMod(ts *testscript.TestScript, neg bool, args []string) {
 		ts.Fatalf("%s unexpectedly has mode %03o", args[1], fileInfo.Mode().Perm())
 	}
 	if !neg && !equal {
-		ts.Fatalf("%s has mode %03o, expected %03o", args[1], fileInfo.Mode().Perm(), fs.FileMode(mode64)&^chezmoitest.Umask)
+		ts.Fatalf(
+			"%s has mode %03o, expected %03o",
+			args[1],
+			fileInfo.Mode().Perm(),
+			fs.FileMode(mode64)&^chezmoitest.Umask,
+		)
 	}
 }
 
@@ -179,6 +178,32 @@ func cmdEdit(ts *testscript.TestScript, neg bool, args []string) {
 	}
 }
 
+// cmdExpandEnv expands environment variables in the given paths.
+func cmdExpandEnv(ts *testscript.TestScript, neg bool, args []string) {
+	if neg {
+		ts.Fatalf("unsupported: ! expandenv")
+	}
+	if len(args) == 0 {
+		ts.Fatalf("usage: expandenv paths...")
+	}
+	for _, arg := range args {
+		filename := ts.MkAbs(arg)
+		data, err := os.ReadFile(filename)
+		if err != nil {
+			ts.Fatalf("%s: %v", filename, err)
+		}
+		data = envVarRx.ReplaceAllFunc(data, func(key []byte) []byte {
+			if value := ts.Getenv(string(bytes.TrimPrefix(key, []byte{'$'}))); value != "" {
+				return []byte(value)
+			}
+			return key
+		})
+		if err := os.WriteFile(filename, data, 0o666); err != nil {
+			ts.Fatalf("%s: %v", filename, err)
+		}
+	}
+}
+
 // cmdHTTPD starts an HTTP server serving files from the given directory and
 // sets the HTTPD_URL environment variable to the URL of the server.
 func cmdHTTPD(ts *testscript.TestScript, neg bool, args []string) {
@@ -193,7 +218,24 @@ func cmdHTTPD(ts *testscript.TestScript, neg bool, args []string) {
 	ts.Setenv("HTTPD_URL", server.URL)
 }
 
-// cmdIsSymlink returns true if all of its arguments are symlinks.
+// cmdIsDir succeeds if all of its arguments are directories.
+func cmdIsDir(ts *testscript.TestScript, neg bool, args []string) {
+	for _, arg := range args {
+		filename := ts.MkAbs(arg)
+		fileInfo, err := os.Lstat(filename)
+		if err != nil {
+			ts.Fatalf("%s: %v", arg, err)
+		}
+		switch isDir := fileInfo.IsDir(); {
+		case isDir && neg:
+			ts.Fatalf("%s is a directory", arg)
+		case !isDir && !neg:
+			ts.Fatalf("%s is not a directory", arg)
+		}
+	}
+}
+
+// cmdIsSymlink succeeds if all of its arguments are symlinks.
 func cmdIsSymlink(ts *testscript.TestScript, neg bool, args []string) {
 	for _, arg := range args {
 		filename := ts.MkAbs(arg)
@@ -210,6 +252,23 @@ func cmdIsSymlink(ts *testscript.TestScript, neg bool, args []string) {
 	}
 }
 
+// cmdLExists succeeds if all if its arguments exist, without following symlinks.
+func cmdLExists(ts *testscript.TestScript, neg bool, args []string) {
+	if len(args) == 0 {
+		ts.Fatalf("usage: exists file...")
+	}
+
+	for _, arg := range args {
+		filename := ts.MkAbs(arg)
+		switch _, err := os.Lstat(filename); {
+		case err == nil && neg:
+			ts.Fatalf("%s unexpectedly exists", filename)
+		case errors.Is(err, fs.ErrNotExist) && !neg:
+			ts.Fatalf("%s does not exist", filename)
+		}
+	}
+}
+
 // cmdMkFile creates empty files.
 func cmdMkFile(ts *testscript.TestScript, neg bool, args []string) {
 	if neg {
@@ -233,7 +292,7 @@ func cmdMkFile(ts *testscript.TestScript, neg bool, args []string) {
 		case !errors.Is(err, fs.ErrNotExist):
 			ts.Fatalf("%s: %v", arg, err)
 		}
-		if err := os.WriteFile(filename, nil, perm); err != nil {
+		if err := writeNewFile(filename, nil, perm); err != nil {
 			ts.Fatalf("%s: %v", arg, err)
 		}
 	}
@@ -249,12 +308,12 @@ func cmdMkAgeConfig(ts *testscript.TestScript, neg bool, args []string) {
 	}
 	symmetric := len(args) == 1 && args[0] == "-symmetric"
 	homeDir := ts.Getenv("HOME")
-	ts.Check(os.MkdirAll(homeDir, 0o777))
+	ts.Check(os.MkdirAll(homeDir, fs.ModePerm))
 	identityFile := filepath.Join(homeDir, "key.txt")
 	recipient, err := chezmoitest.AgeGenerateKey(ts.MkAbs(identityFile))
 	ts.Check(err)
 	configFile := filepath.Join(homeDir, ".config", "chezmoi", "chezmoi.toml")
-	ts.Check(os.MkdirAll(filepath.Dir(configFile), 0o777))
+	ts.Check(os.MkdirAll(filepath.Dir(configFile), fs.ModePerm))
 	lines := []string{
 		`encryption = "age"`,
 		`[age]`,
@@ -265,7 +324,7 @@ func cmdMkAgeConfig(ts *testscript.TestScript, neg bool, args []string) {
 	} else {
 		lines = append(lines, `    recipient = `+strconv.Quote(recipient))
 	}
-	ts.Check(os.WriteFile(configFile, []byte(chezmoitest.JoinLines(lines...)), 0o666))
+	ts.Check(writeNewFile(configFile, []byte(chezmoitest.JoinLines(lines...)), 0o666))
 }
 
 // cmdMkGitConfig makes a .gitconfig file in the home directory.
@@ -280,8 +339,8 @@ func cmdMkGitConfig(ts *testscript.TestScript, neg bool, args []string) {
 	if len(args) > 0 {
 		path = ts.MkAbs(args[0])
 	}
-	ts.Check(os.MkdirAll(filepath.Dir(path), 0o777))
-	ts.Check(os.WriteFile(path, []byte(chezmoitest.JoinLines(
+	ts.Check(os.MkdirAll(filepath.Dir(path), fs.ModePerm))
+	ts.Check(writeNewFile(path, []byte(chezmoitest.JoinLines(
 		`[core]`,
 		`    autocrlf = false`,
 		`[init]`,
@@ -317,14 +376,14 @@ func cmdMkGPGConfig(ts *testscript.TestScript, neg bool, args []string) {
 		ts.Check(os.Chmod(gpgHomeDir, 0o700))
 	}
 
-	command, err := exec.LookPath("gpg")
+	command, err := chezmoi.LookPath("gpg")
 	ts.Check(err)
 
 	key, passphrase, err := chezmoitest.GPGGenerateKey(command, gpgHomeDir)
 	ts.Check(err)
 
 	configFile := filepath.Join(ts.Getenv("HOME"), ".config", "chezmoi", "chezmoi.toml")
-	ts.Check(os.MkdirAll(filepath.Dir(configFile), 0o777))
+	ts.Check(os.MkdirAll(filepath.Dir(configFile), fs.ModePerm))
 	lines := []string{
 		`encryption = "gpg"`,
 		`[gpg]`,
@@ -340,7 +399,7 @@ func cmdMkGPGConfig(ts *testscript.TestScript, neg bool, args []string) {
 	} else {
 		lines = append(lines, `    recipient = "`+key+`"`)
 	}
-	ts.Check(os.WriteFile(configFile, []byte(chezmoitest.JoinLines(lines...)), 0o666))
+	ts.Check(writeNewFile(configFile, []byte(chezmoitest.JoinLines(lines...)), 0o666))
 }
 
 // cmdMkHomeDir makes and populates a home directory.
@@ -358,18 +417,18 @@ func cmdMkHomeDir(ts *testscript.TestScript, neg bool, args []string) {
 	workDir := ts.Getenv("WORK")
 	relPath, err := filepath.Rel(workDir, path)
 	ts.Check(err)
-	if err := vfst.NewBuilder().Build(vfs.NewPathFS(vfs.OSFS, workDir), map[string]interface{}{
-		relPath: map[string]interface{}{
+	if err := vfst.NewBuilder().Build(vfs.NewPathFS(vfs.OSFS, workDir), map[string]any{
+		relPath: map[string]any{
 			".create": "# contents of .create\n",
-			".dir": map[string]interface{}{
+			".dir": map[string]any{
 				"file": "# contents of .dir/file\n",
-				"subdir": map[string]interface{}{
+				"subdir": map[string]any{
 					"file": "# contents of .dir/subdir/file\n",
 				},
 			},
 			".empty": "",
 			".executable": &vfst.File{
-				Perm:     0o777,
+				Perm:     fs.ModePerm,
 				Contents: []byte("# contents of .executable\n"),
 			},
 			".file": "# contents of .file\n",
@@ -404,12 +463,12 @@ func cmdMkSourceDir(ts *testscript.TestScript, neg bool, args []string) {
 	workDir := ts.Getenv("WORK")
 	relPath, err := filepath.Rel(workDir, sourceDir)
 	ts.Check(err)
-	err = vfst.NewBuilder().Build(vfs.NewPathFS(vfs.OSFS, workDir), map[string]interface{}{
-		relPath: map[string]interface{}{
+	err = vfst.NewBuilder().Build(vfs.NewPathFS(vfs.OSFS, workDir), map[string]any{
+		relPath: map[string]any{
 			"create_dot_create": "# contents of .create\n",
-			"dot_dir": map[string]interface{}{
+			"dot_dir": map[string]any{
 				"file": "# contents of .dir/file\n",
-				"subdir": map[string]interface{}{
+				"exact_subdir": map[string]any{
 					"file": "# contents of .dir/subdir/file\n",
 				},
 			},
@@ -535,7 +594,7 @@ func cmdUNIX2DOS(ts *testscript.TestScript, neg bool, args []string) {
 // goosCondition evaluates cond as a logical OR of GOARCHes or GOOSes enclosed
 // in parentheses, returning true if any of them match.
 func goosCondition(cond string) (result, valid bool) {
-	// Interpret the condition as a logical OR of terms in parantheses.
+	// Interpret the condition as a logical OR of terms in parentheses.
 	if !strings.HasPrefix(cond, "(") || !strings.HasSuffix(cond, ")") {
 		result = false
 		valid = false
@@ -574,7 +633,10 @@ func goosCondition(cond string) (result, valid bool) {
 }
 
 func prependDirToPath(dir, path string) string {
-	return strings.Join(append([]string{dir}, filepath.SplitList(path)...), string(os.PathListSeparator))
+	return strings.Join(
+		append([]string{dir}, filepath.SplitList(path)...),
+		string(os.PathListSeparator),
+	)
 }
 
 func setup(env *testscript.Env) error {
@@ -606,10 +668,10 @@ func setup(env *testscript.Env) error {
 		env.Setenv("SHELL", filepath.Join(binDir, "shell"))
 	}
 
-	root := make(map[string]interface{})
+	root := make(map[string]any)
 	switch runtime.GOOS {
 	case "windows":
-		root["/bin"] = map[string]interface{}{
+		root["/bin"] = map[string]any{
 			// editor.cmd is a non-interactive script that appends "# edited\n"
 			// to the end of each file and creates an empty .edited file in each
 			// directory.
@@ -619,6 +681,7 @@ func setup(env *testscript.Env) error {
 					`:loop`,
 					`IF EXIST %~s1\NUL (`,
 					`	copy /y NUL "%~1\.edited" >NUL`,
+					// FIXME recursively edit all files if in a directory
 					`) ELSE (`,
 					`	echo # edited >> "%~1"`,
 					`)`,
@@ -628,10 +691,9 @@ func setup(env *testscript.Env) error {
 			},
 		}
 	default:
-		root["/bin"] = map[string]interface{}{
+		root["/bin"] = map[string]any{
 			// editor is a non-interactive script that appends "# edited\n" to
-			// the end of each file and creates an empty .edited file in each
-			// directory.
+			// the end of each file.
 			"editor": &vfst.File{
 				Perm: 0o755,
 				Contents: []byte(chezmoitest.JoinLines(
@@ -640,22 +702,15 @@ func setup(env *testscript.Env) error {
 					`for name in $*; do`,
 					`    if [ -d $name ]; then`,
 					`        touch $name/.edited`,
+					`        for filename in $(find $name -type f); do`,
+					`            echo "# edited" >> $filename`,
+					`        done`,
 					`    else`,
 					`        echo "# edited" >> $name`,
 					`    fi`,
 					`done`,
 				)),
 			},
-			// shell is a non-interactive script that appends the directory in
-			// which it was launched to $WORK/shell.log.
-			"shell": &vfst.File{
-				Perm: 0o755,
-				Contents: []byte(chezmoitest.JoinLines(
-					`#!/bin/sh`,
-					``,
-					`pwd >> '`+filepath.Join(env.WorkDir, "shell.log")+`'`,
-				)),
-			},
 		}
 	}
 
@@ -679,3 +734,14 @@ func unix2DOS(data []byte) ([]byte, error) {
 	}
 	return []byte(builder.String()), nil
 }
+
+func writeNewFile(filename string, data []byte, perm fs.FileMode) error {
+	switch _, err := os.Lstat(filename); {
+	case err == nil:
+		return fmt.Errorf("%s: %w", filename, fs.ErrExist)
+	case errors.Is(err, fs.ErrNotExist):
+		return os.WriteFile(filename, data, perm)
+	default:
+		return err
+	}
+}
