@@ -17,33 +17,41 @@ func (c *Config) newMergeAllCmd() *cobra.Command {
 		Short:   "Perform a three-way merge for each modified file",
 		Long:    mustLongHelp("merge-all"),
 		Example: example("merge-all"),
-		RunE:    c.makeRunEWithSourceState(c.runMergeAllCmd),
-		Annotations: map[string]string{
-			modifiesSourceDirectory: "true",
-			requiresSourceDirectory: "true",
-		},
+		RunE:    c.runMergeAllCmd,
+		Annotations: newAnnotations(
+			modifiesSourceDirectory,
+			requiresSourceDirectory,
+		),
 	}
 
 	flags := mergeAllCmd.Flags()
 	flags.BoolVar(&c.mergeAll.init, "init", c.mergeAll.init, "Recreate config file from template")
-	flags.BoolVarP(&c.mergeAll.recursive, "recursive", "r", c.mergeAll.recursive, "Recurse into subdirectories")
+	flags.BoolVarP(
+		&c.mergeAll.recursive,
+		"recursive",
+		"r",
+		c.mergeAll.recursive,
+		"Recurse into subdirectories",
+	)
 
 	return mergeAllCmd
 }
 
-func (c *Config) runMergeAllCmd(cmd *cobra.Command, args []string, sourceState *chezmoi.SourceState) error {
+func (c *Config) runMergeAllCmd(cmd *cobra.Command, args []string) error {
 	var targetRelPaths []chezmoi.RelPath
 	dryRunSystem := chezmoi.NewDryRunSystem(c.destSystem)
 	preApplyFunc := func(
 		targetRelPath chezmoi.RelPath, targetEntryState, lastWrittenEntryState, actualEntryState *chezmoi.EntryState,
 	) error {
-		if !targetEntryState.Equivalent(actualEntryState) {
+		if targetEntryState.Type == chezmoi.EntryStateTypeFile &&
+			!targetEntryState.Equivalent(actualEntryState) {
 			targetRelPaths = append(targetRelPaths, targetRelPath)
 		}
 		return chezmoi.Skip
 	}
 	if err := c.applyArgs(cmd.Context(), dryRunSystem, c.DestDirAbsPath, args, applyArgsOptions{
-		include:      chezmoi.NewEntryTypeSet(chezmoi.EntryTypeFiles),
+		cmd:          cmd,
+		filter:       chezmoi.NewEntryTypeFilter(chezmoi.EntryTypesAll, chezmoi.EntryTypesNone),
 		init:         c.mergeAll.init,
 		recursive:    c.mergeAll.recursive,
 		umask:        c.Umask,
@@ -52,6 +60,11 @@ func (c *Config) runMergeAllCmd(cmd *cobra.Command, args []string, sourceState *
 		return err
 	}
 
+	sourceState, err := c.getSourceState(cmd.Context(), cmd)
+	if err != nil {
+		return err
+	}
+
 	for _, targetRelPath := range targetRelPaths {
 		sourceStateEntry := sourceState.MustEntry(targetRelPath)
 		if err := c.doMerge(targetRelPath, sourceStateEntry); err != nil {
