@@ -14,8 +14,8 @@ import (
 )
 
 type mergeCmdConfig struct {
-	Command string   `mapstructure:"command"`
-	Args    []string `mapstructure:"args"`
+	Command string   `json:"command" mapstructure:"command" yaml:"command"`
+	Args    []string `json:"args"    mapstructure:"args"    yaml:"args"`
 }
 
 func (c *Config) newMergeCmd() *cobra.Command {
@@ -27,18 +27,23 @@ func (c *Config) newMergeCmd() *cobra.Command {
 		Example:           example("merge"),
 		ValidArgsFunction: c.targetValidArgs,
 		RunE:              c.makeRunEWithSourceState(c.runMergeCmd),
-		Annotations: map[string]string{
-			modifiesSourceDirectory: "true",
-			requiresSourceDirectory: "true",
-		},
+		Annotations: newAnnotations(
+			modifiesSourceDirectory,
+			requiresSourceDirectory,
+		),
 	}
 
 	return mergeCmd
 }
 
-func (c *Config) runMergeCmd(cmd *cobra.Command, args []string, sourceState *chezmoi.SourceState) error {
+func (c *Config) runMergeCmd(
+	cmd *cobra.Command,
+	args []string,
+	sourceState *chezmoi.SourceState,
+) error {
 	targetRelPaths, err := c.targetRelPaths(sourceState, args, targetRelPathsOptions{
-		mustBeInSourceState: false,
+		mustBeInSourceState: true,
+		mustBeManaged:       false,
 		recursive:           true,
 	})
 	if err != nil {
@@ -58,7 +63,10 @@ func (c *Config) runMergeCmd(cmd *cobra.Command, args []string, sourceState *che
 // doMerge is the core merge functionality. It invokes the merge tool to do a
 // three-way merge between the destination, source, and target, including
 // transparently decrypting the file in the source state.
-func (c *Config) doMerge(targetRelPath chezmoi.RelPath, sourceStateEntry chezmoi.SourceStateEntry) (err error) {
+func (c *Config) doMerge(
+	targetRelPath chezmoi.RelPath,
+	sourceStateEntry chezmoi.SourceStateEntry,
+) (err error) {
 	sourceAbsPath := c.SourceDirAbsPath.Join(sourceStateEntry.SourceRelPath().RelPath())
 
 	// If the source state entry is an encrypted file, then decrypt it to a
@@ -71,7 +79,9 @@ func (c *Config) doMerge(targetRelPath chezmoi.RelPath, sourceStateEntry chezmoi
 			if plaintextTempDirAbsPath, err = c.tempDir("chezmoi-merge-plaintext"); err != nil {
 				return
 			}
-			plaintextAbsPath = plaintextTempDirAbsPath.Join(sourceStateEntry.SourceRelPath().RelPath())
+			plaintextAbsPath = plaintextTempDirAbsPath.Join(
+				sourceStateEntry.SourceRelPath().RelPath(),
+			)
 			defer func() {
 				err = multierr.Append(err, os.RemoveAll(plaintextAbsPath.String()))
 			}()
@@ -79,6 +89,9 @@ func (c *Config) doMerge(targetRelPath chezmoi.RelPath, sourceStateEntry chezmoi
 			if plaintext, err = sourceStateFile.Contents(); err != nil {
 				return
 			}
+			if err = chezmoi.MkdirAll(c.baseSystem, plaintextAbsPath.Dir(), 0o700); err != nil {
+				return
+			}
 			if err = c.baseSystem.WriteFile(plaintextAbsPath, plaintext, 0o600); err != nil {
 				return
 			}
@@ -141,7 +154,7 @@ func (c *Config) doMerge(targetRelPath chezmoi.RelPath, sourceStateEntry chezmoi
 	// merge.args config option replaced all arguments to the merge command.
 	//
 	// Work around this by looking for any templates in merge.args. An arg
-	// is considered a template if, after execution as as template, it is
+	// is considered a template if, after execution as a template, it is
 	// not equal to the original arg.
 	anyTemplateArgs := false
 	for i, arg := range c.Merge.Args {
