@@ -2,6 +2,7 @@ package cmd
 
 import (
 	"bytes"
+	"fmt"
 	"io/fs"
 	"sort"
 
@@ -11,45 +12,51 @@ import (
 )
 
 type reAddCmdConfig struct {
-	exclude   *chezmoi.EntryTypeSet
-	include   *chezmoi.EntryTypeSet
-	recursive bool
+	filter *chezmoi.EntryTypeFilter
 }
 
 func (c *Config) newReAddCmd() *cobra.Command {
 	reAddCmd := &cobra.Command{
-		Use:               "re-add [targets...]",
+		Use:               "re-add",
 		Short:             "Re-add modified files",
 		Long:              mustLongHelp("re-add"),
 		Example:           example("re-add"),
 		ValidArgsFunction: c.targetValidArgs,
-		Args:              cobra.ArbitraryArgs,
+		Args:              cobra.NoArgs,
 		RunE:              c.makeRunEWithSourceState(c.runReAddCmd),
-		Annotations: map[string]string{
-			modifiesSourceDirectory: "true",
-			persistentStateMode:     persistentStateModeReadWrite,
-			requiresSourceDirectory: "true",
-		},
+		Annotations: newAnnotations(
+			modifiesSourceDirectory,
+			persistentStateModeReadWrite,
+			requiresSourceDirectory,
+		),
 	}
 
 	flags := reAddCmd.Flags()
-	flags.VarP(c.reAdd.exclude, "exclude", "x", "Exclude entry types")
-	flags.VarP(c.reAdd.include, "include", "i", "Include entry types")
-	flags.BoolVarP(&c.reAdd.recursive, "recursive", "r", c.reAdd.recursive, "Recurse into subdirectories")
+	flags.VarP(c.reAdd.filter.Exclude, "exclude", "x", "Exclude entry types")
+	flags.VarP(c.reAdd.filter.Include, "include", "i", "Include entry types")
+
+	registerExcludeIncludeFlagCompletionFuncs(reAddCmd)
 
 	return reAddCmd
 }
 
-func (c *Config) runReAddCmd(cmd *cobra.Command, args []string, sourceState *chezmoi.SourceState) error {
+func (c *Config) runReAddCmd(
+	cmd *cobra.Command,
+	args []string,
+	sourceState *chezmoi.SourceState,
+) error {
 	var targetRelPaths chezmoi.RelPaths
 	sourceStateEntries := make(map[chezmoi.RelPath]chezmoi.SourceStateEntry)
-	_ = sourceState.ForEach(func(targetRelPath chezmoi.RelPath, sourceStateEntry chezmoi.SourceStateEntry) error {
-		targetRelPaths = append(targetRelPaths, targetRelPath)
-		sourceStateEntries[targetRelPath] = sourceStateEntry
-		return nil
-	})
+	_ = sourceState.ForEach(
+		func(targetRelPath chezmoi.RelPath, sourceStateEntry chezmoi.SourceStateEntry) error {
+			targetRelPaths = append(targetRelPaths, targetRelPath)
+			sourceStateEntries[targetRelPath] = sourceStateEntry
+			return nil
+		},
+	)
 	sort.Sort(targetRelPaths)
 
+TARGET_REL_PATH:
 	for _, targetRelPath := range targetRelPaths {
 		sourceStateFile, ok := sourceStateEntries[targetRelPath].(*chezmoi.SourceStateFile)
 		if !ok {
@@ -94,14 +101,48 @@ func (c *Config) runReAddCmd(cmd *cobra.Command, args []string, sourceState *che
 			continue
 		}
 
+		if c.interactive {
+			prompt := fmt.Sprintf("Re-add %s", targetRelPath)
+			var choices []string
+			if actualContents != nil || targetContents != nil {
+				choices = append(choices, "diff")
+			}
+			choices = append(choices, choicesYesNoAllQuit...)
+		FOR:
+			for {
+				switch choice, err := c.promptChoice(prompt, choices); {
+				case err != nil:
+					return err
+				case choice == "diff":
+					if err := c.diffFile(
+						targetRelPath,
+						targetContents, targetStateFile.Perm(c.Umask),
+						actualContents, actualStateFile.Perm(),
+					); err != nil {
+						return err
+					}
+				case choice == "yes":
+					break FOR
+				case choice == "no":
+					continue TARGET_REL_PATH
+				case choice == "all":
+					c.interactive = false
+					break FOR
+				case choice == "quit":
+					return chezmoi.ExitCodeError(0)
+				default:
+					panic(fmt.Sprintf("%s: unexpected choice", choice))
+				}
+			}
+		}
+
 		destAbsPathInfos := map[chezmoi.AbsPath]fs.FileInfo{
 			destAbsPath: destAbsPathInfo,
 		}
 		if err := sourceState.Add(c.sourceSystem, c.persistentState, c.destSystem, destAbsPathInfos, &chezmoi.AddOptions{
-			Empty:           sourceStateFile.Attr.Empty,
 			Encrypt:         sourceStateFile.Attr.Encrypted,
 			EncryptedSuffix: c.encryption.EncryptedSuffix(),
-			Include:         c.reAdd.include.Sub(c.reAdd.exclude),
+			Filter:          c.reAdd.filter,
 		}); err != nil {
 			return err
 		}
