@@ -11,19 +11,13 @@ import (
 )
 
 type stateCmdConfig struct {
-	data         stateDataCmdConfig
 	delete       stateDeleteCmdConfig
 	deleteBucket stateDeleteBucketCmdConfig
-	dump         stateDumpCmdConfig
 	get          stateGetCmdConfig
 	getBucket    stateGetBucketCmdConfig
 	set          stateSetCmdConfig
 }
 
-type stateDataCmdConfig struct {
-	format writeDataFormat
-}
-
 type stateDeleteCmdConfig struct {
 	bucket string
 	key    string
@@ -33,10 +27,6 @@ type stateDeleteBucketCmdConfig struct {
 	bucket string
 }
 
-type stateDumpCmdConfig struct {
-	format writeDataFormat
-}
-
 type stateGetCmdConfig struct {
 	bucket string
 	key    string
@@ -44,7 +34,6 @@ type stateGetCmdConfig struct {
 
 type stateGetBucketCmdConfig struct {
 	bucket string
-	format writeDataFormat
 }
 
 type stateSetCmdConfig struct {
@@ -66,12 +55,15 @@ func (c *Config) newStateCmd() *cobra.Command {
 		Short: "Print the raw data in the persistent state",
 		Args:  cobra.NoArgs,
 		RunE:  c.runStateDataCmd,
-		Annotations: map[string]string{
-			persistentStateMode: persistentStateModeReadOnly,
-		},
+		Annotations: newAnnotations(
+			persistentStateModeReadOnly,
+		),
 	}
 	stateDataPersistentFlags := stateDataCmd.PersistentFlags()
-	stateDataPersistentFlags.VarP(&c.state.data.format, "format", "f", "format")
+	stateDataPersistentFlags.VarP(&c.Format, "format", "f", "Output format")
+	if err := stateDataCmd.RegisterFlagCompletionFunc("format", writeDataFormatFlagCompletionFunc); err != nil {
+		panic(err)
+	}
 	stateCmd.AddCommand(stateDataCmd)
 
 	stateDeleteCmd := &cobra.Command{
@@ -79,13 +71,18 @@ func (c *Config) newStateCmd() *cobra.Command {
 		Short: "Delete a value from the persistent state",
 		Args:  cobra.NoArgs,
 		RunE:  c.runStateDeleteCmd,
-		Annotations: map[string]string{
-			persistentStateMode: persistentStateModeReadWrite,
-		},
+		Annotations: newAnnotations(
+			persistentStateModeReadWrite,
+		),
 	}
 	stateDeletePersistentFlags := stateDeleteCmd.PersistentFlags()
-	stateDeletePersistentFlags.StringVar(&c.state.delete.bucket, "bucket", c.state.delete.bucket, "bucket")
-	stateDeletePersistentFlags.StringVar(&c.state.delete.key, "key", c.state.delete.key, "key")
+	stateDeletePersistentFlags.StringVar(
+		&c.state.delete.bucket,
+		"bucket",
+		c.state.delete.bucket,
+		"Bucket",
+	)
+	stateDeletePersistentFlags.StringVar(&c.state.delete.key, "key", c.state.delete.key, "Key")
 	stateCmd.AddCommand(stateDeleteCmd)
 
 	stateDeleteBucketCmd := &cobra.Command{
@@ -93,12 +90,17 @@ func (c *Config) newStateCmd() *cobra.Command {
 		Short: "Delete a bucket from the persistent state",
 		Args:  cobra.NoArgs,
 		RunE:  c.runStateDeleteBucketCmd,
-		Annotations: map[string]string{
-			persistentStateMode: persistentStateModeReadWrite,
-		},
+		Annotations: newAnnotations(
+			persistentStateModeReadWrite,
+		),
 	}
 	stateDeleteBucketPersistentFlags := stateDeleteBucketCmd.PersistentFlags()
-	stateDeleteBucketPersistentFlags.StringVar(&c.state.deleteBucket.bucket, "bucket", c.state.deleteBucket.bucket, "bucket") //nolint:lll
+	stateDeleteBucketPersistentFlags.StringVar(
+		&c.state.deleteBucket.bucket,
+		"bucket",
+		c.state.deleteBucket.bucket,
+		"Bucket",
+	)
 	stateCmd.AddCommand(stateDeleteBucketCmd)
 
 	stateDumpCmd := &cobra.Command{
@@ -106,12 +108,15 @@ func (c *Config) newStateCmd() *cobra.Command {
 		Short: "Generate a dump of the persistent state",
 		Args:  cobra.NoArgs,
 		RunE:  c.runStateDumpCmd,
-		Annotations: map[string]string{
-			persistentStateMode: persistentStateModeReadOnly,
-		},
+		Annotations: newAnnotations(
+			persistentStateModeReadOnly,
+		),
 	}
 	stateDumpPersistentFlags := stateDumpCmd.PersistentFlags()
-	stateDumpPersistentFlags.VarP(&c.state.dump.format, "format", "f", "format")
+	stateDumpPersistentFlags.VarP(&c.Format, "format", "f", "Output format")
+	if err := stateDumpCmd.RegisterFlagCompletionFunc("format", writeDataFormatFlagCompletionFunc); err != nil {
+		panic(err)
+	}
 	stateCmd.AddCommand(stateDumpCmd)
 
 	stateGetCmd := &cobra.Command{
@@ -119,13 +124,13 @@ func (c *Config) newStateCmd() *cobra.Command {
 		Short: "Get a value from the persistent state",
 		Args:  cobra.NoArgs,
 		RunE:  c.runStateGetCmd,
-		Annotations: map[string]string{
-			persistentStateMode: persistentStateModeReadOnly,
-		},
+		Annotations: newAnnotations(
+			persistentStateModeReadOnly,
+		),
 	}
 	stateGetPersistentFlags := stateGetCmd.PersistentFlags()
-	stateGetPersistentFlags.StringVar(&c.state.get.bucket, "bucket", c.state.get.bucket, "bucket")
-	stateGetPersistentFlags.StringVar(&c.state.get.key, "key", c.state.get.key, "key")
+	stateGetPersistentFlags.StringVar(&c.state.get.bucket, "bucket", c.state.get.bucket, "Bucket")
+	stateGetPersistentFlags.StringVar(&c.state.get.key, "key", c.state.get.key, "Key")
 	stateCmd.AddCommand(stateGetCmd)
 
 	stateGetBucketCmd := &cobra.Command{
@@ -133,13 +138,21 @@ func (c *Config) newStateCmd() *cobra.Command {
 		Short: "Get a bucket from the persistent state",
 		Args:  cobra.NoArgs,
 		RunE:  c.runStateGetBucketCmd,
-		Annotations: map[string]string{
-			persistentStateMode: persistentStateModeReadOnly,
-		},
+		Annotations: newAnnotations(
+			persistentStateModeReadOnly,
+		),
 	}
 	stateGetBucketPersistentFlags := stateGetBucketCmd.PersistentFlags()
-	stateGetBucketPersistentFlags.StringVar(&c.state.getBucket.bucket, "bucket", c.state.getBucket.bucket, "bucket")
-	stateGetBucketPersistentFlags.VarP(&c.state.getBucket.format, "format", "f", "format")
+	stateGetBucketPersistentFlags.StringVar(
+		&c.state.getBucket.bucket,
+		"bucket",
+		c.state.getBucket.bucket,
+		"bucket",
+	)
+	stateGetBucketPersistentFlags.VarP(&c.Format, "format", "f", "Output format")
+	if err := stateGetBucketCmd.RegisterFlagCompletionFunc("format", writeDataFormatFlagCompletionFunc); err != nil {
+		panic(err)
+	}
 	stateCmd.AddCommand(stateGetBucketCmd)
 
 	stateResetCmd := &cobra.Command{
@@ -147,9 +160,9 @@ func (c *Config) newStateCmd() *cobra.Command {
 		Short: "Reset the persistent state",
 		Args:  cobra.NoArgs,
 		RunE:  c.runStateResetCmd,
-		Annotations: map[string]string{
-			modifiesDestinationDirectory: "true",
-		},
+		Annotations: newAnnotations(
+			modifiesDestinationDirectory,
+		),
 	}
 	stateCmd.AddCommand(stateResetCmd)
 
@@ -158,14 +171,14 @@ func (c *Config) newStateCmd() *cobra.Command {
 		Short: "Set a value from the persistent state",
 		Args:  cobra.NoArgs,
 		RunE:  c.runStateSetCmd,
-		Annotations: map[string]string{
-			persistentStateMode: persistentStateModeReadWrite,
-		},
+		Annotations: newAnnotations(
+			persistentStateModeReadWrite,
+		),
 	}
 	stateSetPersistentFlags := stateSetCmd.PersistentFlags()
-	stateSetPersistentFlags.StringVar(&c.state.set.bucket, "bucket", c.state.set.bucket, "bucket")
-	stateSetPersistentFlags.StringVar(&c.state.set.key, "key", c.state.set.key, "key")
-	stateSetPersistentFlags.StringVar(&c.state.set.value, "value", c.state.set.value, "value")
+	stateSetPersistentFlags.StringVar(&c.state.set.bucket, "bucket", c.state.set.bucket, "Bucket")
+	stateSetPersistentFlags.StringVar(&c.state.set.key, "key", c.state.set.key, "Key")
+	stateSetPersistentFlags.StringVar(&c.state.set.value, "value", c.state.set.value, "Value")
 	stateCmd.AddCommand(stateSetCmd)
 
 	return stateCmd
@@ -176,7 +189,7 @@ func (c *Config) runStateDataCmd(cmd *cobra.Command, args []string) error {
 	if err != nil {
 		return err
 	}
-	return c.marshal(c.state.data.format, data)
+	return c.marshal(c.Format, data)
 }
 
 func (c *Config) runStateDeleteCmd(cmd *cobra.Command, args []string) error {
@@ -188,11 +201,19 @@ func (c *Config) runStateDeleteBucketCmd(cmd *cobra.Command, args []string) erro
 }
 
 func (c *Config) runStateDumpCmd(cmd *cobra.Command, args []string) error {
-	data, err := chezmoi.PersistentStateData(c.persistentState)
+	data, err := chezmoi.PersistentStateData(c.persistentState, map[string][]byte{
+		"configState":              chezmoi.ConfigStateBucket,
+		"entryState":               chezmoi.EntryStateBucket,
+		"gitHubKeysState":          gitHubKeysStateBucket,
+		"gitHubLatestReleaseState": gitHubLatestReleaseStateBucket,
+		"gitHubLatestTagState":     gitHubLatestTagStateBucket,
+		"gitRepoExternalState":     chezmoi.GitRepoExternalStateBucket,
+		"scriptState":              chezmoi.ScriptStateBucket,
+	})
 	if err != nil {
 		return err
 	}
-	return c.marshal(c.state.dump.format, data)
+	return c.marshal(c.Format, data)
 }
 
 func (c *Config) runStateGetCmd(cmd *cobra.Command, args []string) error {
@@ -204,11 +225,14 @@ func (c *Config) runStateGetCmd(cmd *cobra.Command, args []string) error {
 }
 
 func (c *Config) runStateGetBucketCmd(cmd *cobra.Command, args []string) error {
-	data, err := chezmoi.PersistentStateBucketData(c.persistentState, []byte(c.state.getBucket.bucket))
+	data, err := chezmoi.PersistentStateBucketData(
+		c.persistentState,
+		[]byte(c.state.getBucket.bucket),
+	)
 	if err != nil {
 		return err
 	}
-	return c.marshal(c.state.getBucket.format, data)
+	return c.marshal(c.Format, data)
 }
 
 func (c *Config) runStateResetCmd(cmd *cobra.Command, args []string) error {
@@ -223,11 +247,13 @@ func (c *Config) runStateResetCmd(cmd *cobra.Command, args []string) error {
 		return err
 	}
 	if !c.force {
-		switch choice, err := c.promptChoice(fmt.Sprintf("Remove %s", persistentStateFileAbsPath), []string{"yes", "no"}); {
+		switch choice, err := c.promptChoice(fmt.Sprintf("Remove %s", persistentStateFileAbsPath), choicesYesNoQuit); {
 		case err != nil:
 			return err
 		case choice == "yes":
 		case choice == "no":
+			fallthrough
+		case choice == "quit":
 			return nil
 		}
 	}
@@ -235,5 +261,9 @@ func (c *Config) runStateResetCmd(cmd *cobra.Command, args []string) error {
 }
 
 func (c *Config) runStateSetCmd(cmd *cobra.Command, args []string) error {
-	return c.persistentState.Set([]byte(c.state.set.bucket), []byte(c.state.set.key), []byte(c.state.set.value))
+	return c.persistentState.Set(
+		[]byte(c.state.set.bucket),
+		[]byte(c.state.set.key),
+		[]byte(c.state.set.value),
+	)
 }
