@@ -1,5 +1,4 @@
 //go:build !noupgrade
-// +build !noupgrade
 
 package cmd
 
@@ -21,7 +20,7 @@ import (
 	"strings"
 
 	"github.com/coreos/go-semver/semver"
-	"github.com/google/go-github/v42/github"
+	"github.com/google/go-github/v53/github"
 	"github.com/spf13/cobra"
 	vfs "github.com/twpayne/go-vfs/v4"
 
@@ -93,13 +92,18 @@ func (c *Config) newUpgradeCmd() *cobra.Command {
 		Example: example("upgrade"),
 		Args:    cobra.NoArgs,
 		RunE:    c.runUpgradeCmd,
-		Annotations: map[string]string{
-			runsCommands: "true",
-		},
+		Annotations: newAnnotations(
+			runsCommands,
+		),
 	}
 
 	flags := upgradeCmd.Flags()
-	flags.StringVar(&c.upgrade.executable, "executable", c.upgrade.method, "Set executable to replace")
+	flags.StringVar(
+		&c.upgrade.executable,
+		"executable",
+		c.upgrade.method,
+		"Set executable to replace",
+	)
 	flags.StringVar(&c.upgrade.method, "method", c.upgrade.method, "Set upgrade method")
 	flags.StringVar(&c.upgrade.owner, "owner", c.upgrade.owner, "Set owner")
 	flags.StringVar(&c.upgrade.repo, "repo", c.upgrade.repo, "Set repo")
@@ -113,14 +117,16 @@ func (c *Config) runUpgradeCmd(cmd *cobra.Command, args []string) error {
 
 	var zeroVersion semver.Version
 	if c.version == zeroVersion && !c.force {
-		return errors.New("cannot upgrade dev version to latest released version unless --force is set")
+		return errors.New(
+			"cannot upgrade dev version to latest released version unless --force is set",
+		)
 	}
 
 	httpClient, err := c.getHTTPClient()
 	if err != nil {
 		return err
 	}
-	client := newGitHubClient(ctx, httpClient)
+	client := chezmoi.NewGitHubClient(ctx, httpClient)
 
 	// Get the latest release.
 	rr, _, err := client.Repositories.GetLatestRelease(ctx, c.upgrade.owner, c.upgrade.repo)
@@ -155,7 +161,12 @@ func (c *Config) runUpgradeCmd(cmd *cobra.Command, args []string) error {
 		case err != nil:
 			return err
 		case method == "":
-			return fmt.Errorf("%s/%s: cannot determine upgrade method for %s", runtime.GOOS, runtime.GOARCH, executableAbsPath)
+			return fmt.Errorf(
+				"%s/%s: cannot determine upgrade method for %s",
+				runtime.GOOS,
+				runtime.GOARCH,
+				executableAbsPath,
+			)
 		}
 	}
 	c.logger.Info().
@@ -195,7 +206,7 @@ func (c *Config) runUpgradeCmd(cmd *cobra.Command, args []string) error {
 	// that, otherwise look in $PATH.
 	path := c.upgrade.executable
 	if method != upgradeMethodReplaceExecutable {
-		path, err = exec.LookPath(c.upgrade.repo)
+		path, err = chezmoi.LookPath(c.upgrade.repo)
 		if err != nil {
 			return err
 		}
@@ -206,15 +217,22 @@ func (c *Config) runUpgradeCmd(cmd *cobra.Command, args []string) error {
 	chezmoiVersionCmd.Stdin = os.Stdin
 	chezmoiVersionCmd.Stdout = os.Stdout
 	chezmoiVersionCmd.Stderr = os.Stderr
-	return c.baseSystem.RunIdempotentCmd(chezmoiVersionCmd)
+	return chezmoilog.LogCmdRun(chezmoiVersionCmd)
 }
 
 func (c *Config) brewUpgrade() error {
 	return c.run(chezmoi.EmptyAbsPath, "brew", []string{"upgrade", c.upgrade.repo})
 }
 
-func (c *Config) getChecksums(ctx context.Context, rr *github.RepositoryRelease) (map[string][]byte, error) {
-	name := fmt.Sprintf("%s_%s_checksums.txt", c.upgrade.repo, strings.TrimPrefix(rr.GetTagName(), "v"))
+func (c *Config) getChecksums(
+	ctx context.Context,
+	rr *github.RepositoryRelease,
+) (map[string][]byte, error) {
+	name := fmt.Sprintf(
+		"%s_%s_checksums.txt",
+		c.upgrade.repo,
+		strings.TrimPrefix(rr.GetTagName(), "v"),
+	)
 	releaseAsset := getReleaseAssetByName(rr, name)
 	if releaseAsset == nil {
 		return nil, fmt.Errorf("%s: cannot find release asset", name)
@@ -238,7 +256,7 @@ func (c *Config) getChecksums(ctx context.Context, rr *github.RepositoryRelease)
 }
 
 func (c *Config) downloadURL(ctx context.Context, url string) ([]byte, error) {
-	req, err := http.NewRequestWithContext(ctx, http.MethodGet, url, nil)
+	req, err := http.NewRequestWithContext(ctx, http.MethodGet, url, http.NoBody)
 	if err != nil {
 		return nil, err
 	}
@@ -252,7 +270,7 @@ func (c *Config) downloadURL(ctx context.Context, url string) ([]byte, error) {
 	}
 	if resp.StatusCode != http.StatusOK {
 		_ = resp.Body.Close()
-		return nil, fmt.Errorf("%s: got a non-200 OK response: %d %s", url, resp.StatusCode, resp.Status)
+		return nil, fmt.Errorf("%s: %s", url, resp.Status)
 	}
 	data, err := io.ReadAll(resp.Body)
 	if err != nil {
@@ -270,7 +288,7 @@ func (c *Config) getLibc() (string, error) {
 	// writes to stdout and exits with code 0. On musl libc systems it writes to
 	// stderr and exits with code 1.
 	lddCmd := exec.Command("ldd", "--version")
-	switch output, _ := c.baseSystem.IdempotentCmdCombinedOutput(lddCmd); {
+	switch output, _ := chezmoilog.LogCmdCombinedOutput(lddCmd); {
 	case libcTypeGlibcRx.Match(output):
 		return libcTypeGlibc, nil
 	case libcTypeMuslRx.Match(output):
@@ -279,14 +297,18 @@ func (c *Config) getLibc() (string, error) {
 
 	// Second, try getconf GNU_LIBC_VERSION.
 	getconfCmd := exec.Command("getconf", "GNU_LIBC_VERSION")
-	if output, _ := c.baseSystem.IdempotentCmdOutput(getconfCmd); libcTypeGlibcRx.Match(output) {
+	if output, _ := chezmoilog.LogCmdCombinedOutput(getconfCmd); libcTypeGlibcRx.Match(output) {
 		return libcTypeGlibc, nil
 	}
 
 	return "", errors.New("unable to determine libc")
 }
 
-func (c *Config) getPackageFilename(packageType string, version *semver.Version, os, arch string) (string, error) {
+func (c *Config) getPackageFilename(
+	packageType string,
+	version *semver.Version,
+	os, arch string,
+) (string, error) {
 	if archReplacement, ok := archReplacements[packageType][arch]; ok {
 		arch = archReplacement
 	}
@@ -311,20 +333,44 @@ func (c *Config) replaceExecutable(
 	switch {
 	case runtime.GOOS == "windows":
 		archiveFormat = chezmoi.ArchiveFormatZip
-		archiveName = fmt.Sprintf("%s_%s_%s_%s.zip", c.upgrade.repo, releaseVersion, runtime.GOOS, runtime.GOARCH)
+		archiveName = fmt.Sprintf(
+			"%s_%s_%s_%s.zip",
+			c.upgrade.repo,
+			releaseVersion,
+			runtime.GOOS,
+			runtime.GOARCH,
+		)
 	case runtime.GOOS == "linux" && runtime.GOARCH == "amd64":
 		archiveFormat = chezmoi.ArchiveFormatTarGz
 		var libc string
 		if libc, err = c.getLibc(); err != nil {
 			return
 		}
-		archiveName = fmt.Sprintf("%s_%s_%s-%s_%s.tar.gz", c.upgrade.repo, releaseVersion, runtime.GOOS, libc, runtime.GOARCH)
+		archiveName = fmt.Sprintf(
+			"%s_%s_%s-%s_%s.tar.gz",
+			c.upgrade.repo,
+			releaseVersion,
+			runtime.GOOS,
+			libc,
+			runtime.GOARCH,
+		)
 	case runtime.GOOS == "linux" && runtime.GOARCH == "386":
 		archiveFormat = chezmoi.ArchiveFormatTarGz
-		archiveName = fmt.Sprintf("%s_%s_%s_i386.tar.gz", c.upgrade.repo, releaseVersion, runtime.GOOS)
+		archiveName = fmt.Sprintf(
+			"%s_%s_%s_i386.tar.gz",
+			c.upgrade.repo,
+			releaseVersion,
+			runtime.GOOS,
+		)
 	default:
 		archiveFormat = chezmoi.ArchiveFormatTarGz
-		archiveName = fmt.Sprintf("%s_%s_%s_%s.tar.gz", c.upgrade.repo, releaseVersion, runtime.GOOS, runtime.GOARCH)
+		archiveName = fmt.Sprintf(
+			"%s_%s_%s_%s.tar.gz",
+			c.upgrade.repo,
+			releaseVersion,
+			runtime.GOOS,
+			runtime.GOARCH,
+		)
 	}
 	releaseAsset := getReleaseAssetByName(rr, archiveName)
 	if releaseAsset == nil {
@@ -403,7 +449,12 @@ func (c *Config) upgradePackage(
 		}
 
 		// Find the release asset.
-		packageFilename, err := c.getPackageFilename(packageType, version, runtime.GOOS, runtime.GOARCH)
+		packageFilename, err := c.getPackageFilename(
+			packageType,
+			version,
+			runtime.GOOS,
+			runtime.GOARCH,
+		)
 		if err != nil {
 			return err
 		}
@@ -450,7 +501,12 @@ func (c *Config) upgradePackage(
 	}
 }
 
-func (c *Config) verifyChecksum(ctx context.Context, rr *github.RepositoryRelease, name string, data []byte) error {
+func (c *Config) verifyChecksum(
+	ctx context.Context,
+	rr *github.RepositoryRelease,
+	name string,
+	data []byte,
+) error {
 	checksums, err := c.getChecksums(ctx, rr)
 	if err != nil {
 		return err
@@ -462,7 +518,10 @@ func (c *Config) verifyChecksum(ctx context.Context, rr *github.RepositoryReleas
 	checksum := sha256.Sum256(data)
 	if !bytes.Equal(checksum[:], expectedChecksum) {
 		return fmt.Errorf(
-			"%s: checksum failed (want %s, got %s)", name, hex.EncodeToString(expectedChecksum), hex.EncodeToString(checksum[:]),
+			"%s: checksum failed (want %s, got %s)",
+			name,
+			hex.EncodeToString(expectedChecksum),
+			hex.EncodeToString(checksum[:]),
 		)
 	}
 	return nil
@@ -520,7 +579,7 @@ func getUpgradeMethod(fileSystem vfs.Stater, executableAbsPath chezmoi.AbsPath)
 		case 0:
 			method := upgradeMethodUpgradePackage
 			if uid != 0 {
-				if _, err := exec.LookPath("sudo"); err == nil {
+				if _, err := chezmoi.LookPath("sudo"); err == nil {
 					method = upgradeMethodSudoPrefix + method
 				}
 			}
@@ -528,7 +587,10 @@ func getUpgradeMethod(fileSystem vfs.Stater, executableAbsPath chezmoi.AbsPath)
 		case uid:
 			return upgradeMethodReplaceExecutable, nil
 		default:
-			return "", fmt.Errorf("%s: cannot upgrade executable owned by non-current non-root user", executableAbsPath)
+			return "", fmt.Errorf(
+				"%s: cannot upgrade executable owned by non-current non-root user",
+				executableAbsPath,
+			)
 		}
 	case "openbsd":
 		return upgradeMethodReplaceExecutable, nil
@@ -539,7 +601,7 @@ func getUpgradeMethod(fileSystem vfs.Stater, executableAbsPath chezmoi.AbsPath)
 
 // getPackageType returns the distributions package type based on is OS release.
 func getPackageType(system chezmoi.System) (string, error) {
-	osRelease, err := chezmoi.OSRelease(system)
+	osRelease, err := chezmoi.OSRelease(system.UnderlyingFS())
 	if err != nil {
 		return packageTypeNone, err
 	}
@@ -555,7 +617,11 @@ func getPackageType(system chezmoi.System) (string, error) {
 			}
 		}
 	}
-	err = fmt.Errorf("could not determine package type (ID=%q, ID_LIKE=%q)", osRelease["ID"], osRelease["ID_LIKE"])
+	err = fmt.Errorf(
+		"could not determine package type (ID=%q, ID_LIKE=%q)",
+		osRelease["ID"],
+		osRelease["ID_LIKE"],
+	)
 	return packageTypeNone, err
 }
 
